import matplotlib
from gensim import corpora
import pickle
import gensim

class LDAManager:
    def __init__(self, numTopics, numWords):
        name = 'LDA'
        self.numTopics = numTopics
        self.numWords = numWords

    def createDictionary(self, text_data):
        dictionary = corpora.Dictionary(text_data)
        corpus = [dictionary.doc2bow(text) for text in text_data]

        pickle.dump(corpus, open('corpus.pkl', 'wb'))
        dictionary.save('dictionary.gensim')

        return corpus, dictionary

    def run(self, text_data):
        corpus, dictionary = self.createDictionary(text_data)
        tf_idf = gensim.models.TfidfModel(corpus)
        corpus_tfidf = tf_idf[corpus]
        ldamodel = gensim.models.ldamodel.LdaModel(corpus=corpus,
                                                   num_topics=self.numTopics,
                                                   id2word=dictionary,
                                                   passes=15,
                                                   iterations=1500,
                                                   alpha='symmetric',
                                                   per_word_topics=True
                                                   )

        # Get topic weights and dominant topics ------------
        from sklearn.manifold import TSNE
        from bokeh.plotting import figure, output_file, show
        from bokeh.models import Label
        import pandas as pd
        import numpy as np
        import matplotlib.colors as mcolors

        # Get topic weights
        topic_weights = []
        for i, row_list in enumerate(ldamodel[corpus]):
            #print("XXXX " + str(i) + " : " + str(row_list[0]))
            topic_weights.append([w for i, w in row_list[0]])

        # Array of topic weights
        arr = pd.DataFrame(topic_weights).fillna(0).values

        # Keep the well separated points (optional)
        arr = arr[np.amax(arr, axis=1) > 0.35]

        # Dominant topic number in each doc
        topic_num = np.argmax(arr, axis=1)
        print(str(topic_num) + " :: " + str(arr))

        # tSNE Dimension Reduction
        tsne_model = TSNE(n_components=2, verbose=1, random_state=0, angle=.99, init='pca')
        tsne_lda = tsne_model.fit_transform(arr)

        # Plot the Topic Clusters using Bokeh
        n_topics = 4
        mycolors = np.array([color for name, color in matplotlib.colors.cnames.items()])
        plot = figure(title="t-SNE Clustering of {} LDA Topics".format(n_topics),
                      plot_width=900, plot_height=700)
        plot.scatter(x=tsne_lda[:, 0], y=tsne_lda[:, 1], color=mycolors[topic_num])
        show(plot)

        for topic_id in range(ldamodel.num_topics):
            topk = ldamodel.show_topic(topic_id, 10)
            topk_words = [w for w, _ in topk]
            print('{}: {}'.format(topic_id, ' '.join(topk_words)))

        ldamodel.save('model5.gensim')
        topics = ldamodel.print_topics(self.numWords)
        with io.open("topic_results.txt", 'w', encoding='utf8') as f:
            for topic in topics:
                #print(topic)
                f.write(str(topic) + "\n")

        doc_count = 0
        with io.open("doc_topic_results.txt", 'w', encoding='utf8') as f:
            for _doc in corpus:
                document_topics = ldamodel.get_document_topics(_doc)
                doc_dist = ''
                for aTuple in document_topics:
                    doc_dist += str(aTuple[0]) + " : " + str(aTuple[1]) + " "
                #print(str(text_data[doc_count]) + " " + doc_dist + "\n")
                f.write(str(text_data[doc_count]) + " " + doc_dist  + "\n")
                doc_count += 1

if __name__ == '__main__':
    import pyTextMiner as ptm
    import io
    import nltk

    #corpus = ptm.CorpusFromFile('../data/donald.txt')
    corpus = ptm.CorpusFromFieldDelimitedFile('../data/donald.txt', 2)
    pipeline = ptm.Pipeline(ptm.splitter.NLTK(),
                            ptm.tokenizer.Komoran(),
                            ptm.helper.POSFilter('NN*'),
                            ptm.helper.SelectWordOnly(),
                            ptm.helper.StopwordFilter(file='../stopwords/stopwordsKor.txt'),
                            ptm.ngram.NGramTokenizer(1,2))

    result = pipeline.processCorpus(corpus)

    id = 0
    text_data = []
    for doc in result:
        new_doc = []
        for sent in doc:
            for _str in sent:
                if len(_str) > 0:
                    new_doc.append(_str)
        text_data.append(new_doc)
        id += 1

    numTopics = 5
    numWords = 15
    LDAManager(numTopics, numWords).run(text_data)




import datetime
import re
import time
import calendar


class DateUtil(object):
    DATE_START = '2000-01-01'  # 이전의 데이터는 무시함.
    DATE_END = '2016-11-30'

    DATETIME_START = '2000-01-01 23:59:59'  # 이전의 데이터는 무시함.
    DATETIME_END = '2016-09-30 23:59:59'

    @staticmethod
    def yyyymmdd2mysql_date(yyyymmdd):
        if len(yyyymmdd) == 8:
            return '%4d-%02d-%02d' % (int(yyyymmdd[:4]), int(yyyymmdd[4:6]), int(yyyymmdd[6:]))
        else:
            return ''

    @staticmethod
    def mysql_date2yyyymmdd(mysql_date: str):
        tokens = mysql_date.split('-')
        if len(mysql_date) == 10 and len(tokens) == 3:
            return '%4d%02d%02d' % (int(tokens[0]), int(tokens[1]), int(tokens[2]))
        else:
            return ''

    @staticmethod
    def mysql_date2date(mysql_date: str):
        tokens = mysql_date.split('-')
        if len(mysql_date) == 10 and len(tokens) == 3:
            return datetime.date(int(tokens[0]), int(tokens[1]), int(tokens[2]))
        else:
            return datetime.date.today()

    @staticmethod
    def secs_to_string(secs):  # @ReservedAssignment
        secs = int(secs)
        mins, secs = divmod(secs, 60)
        hours, mins = divmod(mins, 60)
        days, hours = divmod(hours, 24)
        if days > 0:
            return '%ddays %02d:%02d:%02d' % (days, hours, mins, secs)
        else:
            return '%02d:%02d:%02d' % (hours, mins, secs)

    @staticmethod
    def millisecs_to_string(_secs):  # @ReservedAssignment
        secs = int(_secs)
        mili_secs = float(_secs) - secs
        mins, secs = divmod(secs, 60)
        hours, mins = divmod(mins, 60)
        days, hours = divmod(hours, 24)
        if days > 0:
            return '%ddays %02d:%02d:%02d%s' % (days, hours, mins, secs, ('%.4f' % mili_secs)[1:])
        else:
            return '%02d:%02d:%02d%s' % (hours, mins, secs, ('%.4f' % mili_secs)[1:])

    @staticmethod
    def current_datetime_string(datetime_format='%04d-%02d-%02d %02d:%02d:%02d'):
        now = time.localtime()
        return datetime_format % (
            now.tm_year, now.tm_mon, now.tm_mday, now.tm_hour, now.tm_min, now.tm_sec)

    @staticmethod
    def to_datetime_string(_datetime: datetime.datetime, format='%04d-%02d-%02d %02d:%02d:%02d'):
        return format % (
            _datetime.year, _datetime.month, _datetime.day, _datetime.hour, _datetime.minute, _datetime.second)

    @staticmethod
    def to_date_string(_date: datetime.date, format='%04d-%02d-%02d'):
        return format % (_date.year, _date.month, _date.day)

    @staticmethod
    def date2mysql_date(_date: datetime.date):
        return DateUtil.to_date_string(_date)

    @staticmethod
    def datetime2yyyymmdd(_datetime: datetime.datetime):
        return '%04d%02d%02d' % (_datetime.year, _datetime.month, _datetime.day)

    @staticmethod
    def date2yyyymmdd(_datetime: datetime.date):
        return '%04d%02d%02d' % (_datetime.year, _datetime.month, _datetime.day)

    @staticmethod
    def current_date_string():
        now = time.localtime()
        return '%04d-%02d-%02d' % (now.tm_year, now.tm_mon, now.tm_mday)

    @staticmethod
    def current_yyyymm():
        now = time.localtime()
        return '%04d%02d' % (now.tm_year, now.tm_mon)

    @staticmethod
    def current_yyyymmdd():
        now = time.localtime()
        return '%04d%02d%02d' % (now.tm_year, now.tm_mon, now.tm_mday)

    @staticmethod
    def current_yyyymmddhhmm():
        now = time.localtime()
        return '%04d%02d%02d%02d%02d' % (now.tm_year, now.tm_mon, now.tm_mday, now.tm_hour, now.tm_min)

    @staticmethod
    def current_yyyymmdd_hhmm():
        now = time.localtime()
        return '%04d%02d%02d_%02d%02d' % (now.tm_year, now.tm_mon, now.tm_mday, now.tm_hour, now.tm_min)

    @staticmethod
    def current_yyyymmdd_hhmmss():
        now = time.localtime()
        return '%04d%02d%02d_%02d%02d%02d' % (now.tm_year, now.tm_mon, now.tm_mday, now.tm_hour, now.tm_min, now.tm_sec)

    @staticmethod
    def current_yyyymmddhhmmss():
        now = time.localtime()
        return '%04d%02d%02d%02d%02d%02d' % (now.tm_year, now.tm_mon, now.tm_mday, now.tm_hour, now.tm_min, now.tm_sec)

    @staticmethod
    def current_hhmmss():
        now = time.localtime()
        return '%d:%d:%d' % (now.tm_hour, now.tm_min, now.tm_sec)

    @staticmethod
    def string_to_unixtime(date_str, time_format='%Y-%m-%d %H:%M:%S'):
        date = DateUtil.string_to_datetime(date_str, time_format)
        return int(date.timestamp())

    @staticmethod
    def current_hhmm00():
        now = time.localtime()
        return '%d:%d:00' % (now.tm_hour, now.tm_min)

    @staticmethod
    def current_millisecs():
        return int(round(time.time() * 1000))

    @staticmethod
    def current_microsecs():
        return int(round(time.time() * 1000000))

    @staticmethod
    def current_unixtimestamp():
        return int(time.time())

    @staticmethod
    def is_mysql_date_format(date_str):
        try:
            DateUtil.string_to_date(date_str)
            return True
        except:
            return False

    @staticmethod
    def is_mysql_datetime_format(date_str):
        try:
            DateUtil.string_to_datetime(date_str)
            return True
        except:
            return False

    # @staticmethod
    # def date_to_datetime(date): # date2datetime
    #     return datetime.datetime(*date.timetuple()[:-4])

    @staticmethod
    def string_to_datetime(date_str, time_format='%Y-%m-%d %H:%M:%S'):
        return datetime.datetime.strptime(date_str, time_format)

    @staticmethod
    def string_to_date(date_str, time_format='%Y-%m-%d'):
        return datetime.datetime.strptime(date_str, time_format).date()

    @staticmethod
    def now():
        return datetime.datetime.now()

    @staticmethod
    def today():
        return datetime.datetime.now().date()

    @staticmethod
    def is_valid_date_string(date_string, date_start='1960-01-01', date_end=None):
        if not date_end:
            date_end = DateUtil.current_date_string()

        try:
            match = re.match(
                '^(((\d{4})(-)(0[13578]|10|11|12)(-)(0[1-9]|[12][0-9]|3[01]))|((\d{4})(-)(0[469]|11)(-)([0][1-9]|[12][0-9]|30))|((\d{4})(-)(0[2])(-)(0[1-9]|1[0-9]|2[0-8]))|(([02468][048]00)(-)(02)(-)(29))|(([13579][26]00)(-)(02)(-)(29))|(([0-9][0-9][0][48])(-)(02)(-)(29))|(([0-9][0-9][2468][048])(-)(02)(-)(29))|(([0-9][0-9][13579][26])(-)(02)(-)(29)))$',
                date_string)
            if not match:
                return False

            # time.strptime(datetime_string, "%Y-%m-%d %H:%M:%S")
            if date_start <= date_string <= date_end:
                return True
            else:
                return False
        except ValueError:
            print('ValueError')
            return False

    @staticmethod
    def is_valid_datetime_string(datetime_string, datetime_start='1960-01-01 00:00:00', datetime_end=None, delta: datetime.timedelta = None):
        if not datetime_string or len(datetime_string) == 0:
            return False

        if not datetime_end:
            if delta:
                datetime_end = DateUtil.to_datetime_string(datetime.datetime.now() + delta)
            else:
                datetime_end = DateUtil.current_datetime_string()
        try:
            match = re.match(
                '^(((\d{4})(-)(0[13578]|10|11|12)(-)(0[1-9]|[12][0-9]|3[01]))|((\d{4})(-)(0[469]|11)(-)([0][1-9]|[12][0-9]|30))|((\d{4})(-)(0[2])(-)(0[1-9]|1[0-9]|2[0-8]))|(([02468][048]00)(-)(02)(-)(29))|(([13579][26]00)(-)(02)(-)(29))|(([0-9][0-9][0][48])(-)(02)(-)(29))|(([0-9][0-9][2468][048])(-)(02)(-)(29))|(([0-9][0-9][13579][26])(-)(02)(-)(29)))(\s([0-1][0-9]|2[0-4]):([0-5][0-9]):([0-5][0-9]))$',
                datetime_string)
            if not match:
                return False

            # time.strptime(datetime_string, "%Y-%m-%d %H:%M:%S")
            if datetime_start <= datetime_string <= datetime_end:
                return True
            else:
                return False
        except ValueError:
            print('ValueError')
            return False

    @staticmethod
    def weekday_string(date: datetime.date, lang: str = 'ko'):
        if lang == 'ko':
            WEEK_DAY = ['월', '화', '수', '목', '금', '토', '일']
        else:
            WEEK_DAY = ['MON', 'TUE', 'WED', 'THU', 'FRI', 'SAT', 'SUN']
        return WEEK_DAY[date.weekday()]

    @staticmethod
    def nth_weekday(year: int, month: int, nth=2, weekday='목'):
        """

        :param year: 년
        :param month: 월
        :param nth: 몇번째인지 1 ~ 5
        :param weekday: 월 화 수 목 금 토 일
        :return:
        """
        c = calendar.Calendar(firstweekday=calendar.SUNDAY)
        weekday_from_kor = {
            '일': calendar.SUNDAY, '월': calendar.MONDAY, '화': calendar.TUESDAY, '수': calendar.WEDNESDAY, '목': calendar.THURSDAY, '금': calendar.FRIDAY, '토': calendar.SATURDAY
        }
        monthcal = c.monthdatescalendar(year=year, month=month)
        return [day for week in monthcal for day in week if day.weekday() == weekday_from_kor[weekday] and day.month == month][nth - 1]

    @classmethod
    def date2datetime(cls, date: datetime.date):
        return datetime.datetime.combine(date, datetime.datetime.min.time())

    @classmethod
    def datetime2date(cls, datetime: datetime.datetime):
        return datetime.date()


if __name__ == '__main__':
    print(datetime.date(datetime.datetime.now()))
    # print(DateUtil.nth_weekday(2017, 10, nth=2, weekday='목'))
    # print(type(now), now)
    # before = now - datetime.timedelta(days=5)
    # print(type(before), before)
    # print(DateUtil.to_date_string(before, format='%04d%02d%02d'))
    # print(DateUtil.weekday_string(DateUtil.string_to_date(str(20171010), time_format='%Y%m%d')))
    # print(DateUtil.string_to_datetime('2017-03-16', time_format='%Y-%m-%d'))
    # time = datetime.datetime.time(datetime.datetime.now())
    # print('time:', time)
    # print(time < datetime.time(0, 30))
    # print(datetime.time(11, 30) < time)
    # date = datetime.datetime.now() + datetime.timedelta(days=1)
    # print(DateUtil.to_datetime_string(date))
    # print(DateUtil.to_datetime_string(datetime.now()))
    # print(DateUtil.current_datetime_string())
    # print('1970-01-01 23:59:59', DateUtil.is_valid_datetime_string('1970-01-01 23:59:59', datetime_start='1990-01-01 23:59:59'))
    # print('1970-01-01 23:59:59', DateUtil.is_valid_datetime_string('1970-01-01 23:59:59'))
    # for datetime_string in ['xx', '9141-1-3 7:10:0', '2000-01-01 23:59:59', '1970-01-01 23:59:59', '2016-02-29 14:21:00']:
    #     print(datetime_string, DateUtil.is_valid_datetime_string(datetime_string, datetime_end='1990-01-01 23:59:59'))

    # print(DateUtil.string_to_unixtime('2017-01-01 00:00:00'))
    # print(DateUtil.current_yyyymmdd_hhmm())
    # @formatter:off
    # print(type(DateUtil.today()), DateUtil.today())
    # print(DateUtil.secs_to_string(1000))
    # print(DateUtil.current_time_string(), len(DateUtil.current_time_string()))
    # print(DateUtil.current_yyyymm(), len(DateUtil.current_yyyymm()))
    # print(DateUtil.current_yyyymmddhhmmss(), len(DateUtil.current_yyyymmddhhmmss()))
    # print(DateUtil.current_unixtimestamp(), len(str(DateUtil.current_unixtimestamp())))
    # print(DateUtil.current_millisecs(), len(str(DateUtil.current_millisecs())))
    # print(DateUtil.current_microsecs(), len(str(DateUtil.current_microsecs())))
    # d = DateUtil.string_to_date('2015-10-20 21:30:40')
    # print(type(d), d, '->', d.year, d.month, d.day, d.hour, d.minute, d.second, d.weekday())
    # print(type(datetime.now()))
    # print('%.10f' % time.time()


import json

from pycrfsuite_spacing import PyCRFSuiteSpacing
from pycrfsuite_spacing import TemplateGenerator
from pycrfsuite_spacing import CharacterFeatureTransformer


class BaseSegmentation:
    IN_TYPE = [str]
    OUT_TYPE = [str]


class SegmentationKorean(BaseSegmentation):
    def __init__(self, model=None):
        # model_path = 'demo_model.crfsuite'
        to_feature = CharacterFeatureTransformer(
            TemplateGenerator(
                begin=-2,
                end=2,
                min_range_length=3,
                max_range_length=3)
        )
        self.inst = PyCRFSuiteSpacing(to_feature)
        self.inst.load_tagger(model)

    def __call__(self, *args, **kwargs):
        return self.inst(args[0])



class LSTMSegmentationKorean(BaseSegmentation):
    def __init__(self, model_path='./model'):
        self.model = model_path
        dic_path = self.model + '/' + 'dic.pickle'

        # config
        self.n_steps = 30  # time steps
        self.padd = '\t'  # special padding chracter

    def __call__(self, *args, **kwargs):

        sentence = args[0]
        sentence_size = len(sentence)
        tag_vector = [-1] * (sentence_size + self.n_steps)  # buffer n_steps
        pos = 0

        return None

    def close(self):
        self.sess.close()


class CNNSegmentationKorean(BaseSegmentation):
    def __init__(self, model_file='./model', training_config='', char_file=''):
        self.model_file = model_file
        self.training_config = training_config
        self.char_file = char_file

        #self.model, self.vocab_table = self.load(model_file=model_file, training_config=training_config, char_file=char_file)


    def load(self, model_file='', training_config='', char_file=''):
        with open(training_config, encoding='utf-8') as f:
            config = json.load(f)

        return model, vocab_table

    def predict(self, model, vocab_table, input_str):
        inference = self.get_inference_fn(model, vocab_table)
        result = ''
        return b"".join(result).decode("utf8")

    def get_inference_fn(self, model, vocab_table):
        inference = None
        return inference


    #def __call__(self, *args, **kwargs):
    #    return self.predict(self.model, self.vocab_table, args[0])

def convert_output_to_string(byte_array, model_output):
    strings_result = None
    return strings_result

if __name__ == "__main__":
    model_file = '../../models/checkpoint-12.ckpt'
    training_config = '../../resources/config.json'
    char_file = '../../resources/chars-4996'

    spacing = CNNSegmentationKorean()

    model, vocab_table = spacing.load(model_file=model_file, training_config=training_config, char_file=char_file)
    input_str = '오늘은우울한날이야...ㅜㅜ'
    result = spacing.predict(model, vocab_table, input_str)
    result = result.replace('<s>','')
    result = result.replace('</s>', '')
    print(result)


class WordCounter:
    IN_TYPE = [list, str]
    OUT_TYPE = [list, tuple]

    def __call__(self, *args, **kwargs):
        from collections import Counter
        return list(Counter(args[0]).most_common())




from soynlp.noun import LRNounExtractor_v2

class BaseNounExtraction:
    INPUT=[str]
    OUTPUT=[list,str]

class NounExtractionKorean(BaseNounExtraction):
    def __init__(self,sents):
        self.inst = LRNounExtractor_v2(verbose=False, extract_compound=True)
        self.inst.train(sents)
        self.inst.extract()

    def __call__(self, *args, **kwargs):
        return self.inst.decompose_compound(args[0])


import gzip
import os


class DataFileUtil(object):
    @staticmethod
    def read_list(filepath, gzip_format=False, strip=False):
        if not os.path.exists(filepath):
            raise Exception('"%s" not exists.' % filepath)

        try:
            if gzip_format:
                f = gzip.open(filepath, 'rt', encoding='utf8')
            else:
                f = open(filepath, 'r', encoding='utf8')

            with f:
                if strip:
                    values = [c.strip() for c in f.read().splitlines()]  # e.g. ['ㄱ', 'ㄴ', 'ㄷ', ...]
                else:
                    values = [c for c in f.read().splitlines()]
                return values
        except:
            return []


import math
import numpy as np

def tf_dc(corpus,package):
	labelset = package ["labelset"]
	weights = package ["weights"]
	voca = package ["voca"]

	label_text = {}
	doclen = {}
	dictlist = {}

	for i in corpus:
		labell = i["label"]
		sentence = i["split_sentence"].split()
		if labell not in label_text:
			label_text[labell] = []
		label_text[labell] = label_text[labell] + sentence

	for a in label_text:
		listt = label_text[a]
		doclen[a]= len(listt)
		if a not in dictlist:
			dictlist[a] = {}
		for i in listt:
			if i not in dictlist[a]:
				dictlist[a][i] = 0
			dictlist[a][i] += 1

	for word in voca:
		hlist = np.zeros((len(labelset)))
		#print len(hlist)
		for cate in range(len(labelset)):
			if word in dictlist[labelset[cate]]:
				hlist[cate] = dictlist[labelset[cate]][word]
		#print hlist
		hlist = hlist/np.sum(hlist)
		for i in range(len(hlist)):
			if abs(hlist[i]-0.0)<1e-5:
				hlist[i]=1
		weights[word] = 1.0 + ( np.sum( hlist*np.log2(hlist)) ) / (math.log(len(labelset),2))

	for i in dictlist:
		for j in dictlist[i]:
			dictlist[i][j] = dictlist[i][j] * 1.0 / doclen[i]
			if j in weights and math.isnan(weights[j]) is not True \
					and math.isnan(dictlist[i][j]) is not True:
				dictlist[i][j] = dictlist[i][j] * weights[j]
	package ["labelset"] = labelset
	package ["weights"] = weights
	package ["voca"] = voca
	return dictlist


# -*- coding: utf-8 -*-
import datetime

from selenium import webdriver
import time
import random
from urllib.parse import quote_plus

class NewspaperCollector:
    def __init__(self, **args):
        self.wait_min = args.get('wait_min', 1)
        self.wait_max = args.get('wait_max', 10)
        self.wait_loading = args.get('wait_loading', 60)
        self.browser = None
        self.keyword = None

    def wait(self):
        time.sleep(random.randint(self.wait_min, self.wait_max))

    def collectByKeyword(self, keyword, output, **args):
        self.keyword = keyword
        article_list = set()
        if output:
            outputFile = open(output, 'w')
        maxNum = args.get('maxNum', -1)
        print('Init Chromedriver...')
        #self.browser = webdriver.PhantomJS(executable_path=r"D:\python_workspace\pyTextMiner\selenium_server\phantomjs.exe")
        self.browser=webdriver.Chrome(executable_path='D:\python_workspace\pyTextMiner\selenium_server\chromedriver.exe')
        url = self.getFirstURL(keyword, **args)
        print('Get Page... ' + url)

        self.browser.get(url)
        self.wait()
        curPage = 0
        while 1:
            secs = 0
            while secs < self.wait_loading:
                links = self.getLinkers(self.browser)
                if len(links): break
                print("Wait for page loading...")
                time.sleep(5)
                secs += 5
            if secs >= self.wait_loading: break
            oLen = len(article_list)
            for l in links:
                href = l.get_attribute('href')
                if href not in article_list and outputFile: outputFile.write(href + "\n")
                article_list.add(href)
                print('url ' + url)
                if maxNum > 0 and len(article_list) >= maxNum: break
            if oLen == len(article_list): break
            if maxNum > 0 and len(article_list) >= maxNum: break
            if outputFile: outputFile.flush()
            try:
                curPage += 1
                print(self.browser.current_url)
                self.getNext(self.browser, curPage)
                print('Get Next Page...')
                self.wait()
            except Exception as e:
                print(e)
                break
        self.browser.quit()
        self.browser = None
        if outputFile: outputFile.close()
        return article_list

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.browser: self.browser.quit()
        self.browser = None

class CNNCollector(NewspaperCollector):
    def getFirstURL(self, keyword, **args):
        return "https://edition.cnn.com/search/?q=" + quote_plus(keyword)

    def getLinkers(self, browser):
        return browser.find_elements_by_css_selector('h3.cnn-search__result-headline a')

    def getNext(self, browser, nextPage):
        browser.find_element_by_css_selector('.pagination-arrow-right').click()

class NYTCollector(NewspaperCollector):
    def getFirstURL(self, keyword, **args):
        #base = "https://query.nytimes.com/search/sitesearch/?action=click&contentCollection&region=TopBar&WT.nav=searchWidget&module=SearchSubmit&pgtype=Homepage#/"
        base="https://www.nytimes.com/search?query="
        time = args.get('time', 'since1851')
        if time == 'since1851':
            time = 'startDate=18510101'
        elif time == '24hours':
            x = datetime.datetime.today()
            a_x = datetime.datetime.today() - datetime.timedelta(days=1)
            time = 'endDate='+str(x.year) + str(x.month) + str(x.day) + '&startDate=' +str(a_x.year) + str(a_x.month) + str(a_x.day)
        elif time == '7days':
            x = datetime.datetime.today()
            a_x = datetime.datetime.today() - datetime.timedelta(days=7)
            time = 'endDate='+str(x.year) + str(x.month) + str(x.day) + '&startDate=' +str(a_x.year) + str(a_x.month) + str(a_x.day)
        elif time == '30days':
            x = datetime.datetime.today()
            a_x = datetime.datetime.today() - datetime.timedelta(days=30)
            time = 'endDate='+str(x.year) + str(x.month) + str(x.day) + '&startDate=' +str(a_x.year) + str(a_x.month) + str(a_x.day)
        elif time == '12months':
            x = datetime.datetime.today()
            a_x = datetime.datetime.today() - datetime.timedelta(months=12)
            time = 'endDate='+str(x.year) + str(x.month) + str(x.day) + '&startDate=' +str(a_x.year) + str(a_x.month) + str(a_x.day)

        result_type = args.get('resultType', 'all')
        if (result_type == "all"):
            return base + quote_plus(keyword) + "&" + time
        else:
            return base + quote_plus(keyword) + "/" + time + "&type=article"

    def getLinkers(self, browser):
        #li.css-1l4w6pd
        return browser.find_elements_by_css_selector('li.css-1l4w6pd div a')

    def getNext(self, browser, nextPage):
        #data-testid="search-show-more-button
        browser.find_element_by_css_selector('[data-testid="search-show-more-button"]').click()

class WPCollector(NewspaperCollector):
    def getFirstURL(self, keyword, **args):
        return "https://www.washingtonpost.com/newssearch/?query=" + quote_plus(keyword) + "#page-1"

    def getLinkers(self, browser):
        return browser.find_elements_by_css_selector('.pb-feed-item a.ng-binding')

    def getNext(self, browser, nextPage):
        browser.get("https://www.washingtonpost.com/newssearch/?query=" + quote_plus(self.keyword) + ("#page-%d" % (nextPage+1)))

class WSJCollector(NewspaperCollector):
    def getFirstURL(self,keyword,**args):
        from datetime import date
        today = date.today()
        minDate = "%04d/%02d/%02d" % (today.year-4, today.month, today.day)
        maxDate = "%04d/%02d/%02d" % (today.year, today.month, today.day)
        return "https://www.wsj.com/search/term.html?KEYWORDS="+quote_plus(keyword)  +"&min-date="+minDate+"&max-date="+maxDate+"&daysback=4y&isAdvanced=true&andor=AND&sort=date-desc&source=wsjarticle,wsjblogs,wsjvideo,sitesearch"


    def getLinkers(self, browser):
        return browser.find_elements_by_css_selector('h3.headline a')

    def getNext(self, browser, nextPage):
        browser.find_element_by_css_selector('.next-page a').click()

class ForbesCollector(NewspaperCollector):
    def collectByKeyword(self, keyword, output, **args):
        import json
        import urllib.request
        self.keyword = keyword
        article_list = []
        outputFile = None
        if output:
            outputFile = open(output, 'w')
        maxNum = args.get('maxNum', -1)
        curPage = 0
        while 1:
            response = urllib.request.urlopen('https://www.forbes.com/forbesapi/search/all.json?limit=100&orfilters=&query=%s&retrievedfields=author,authors,blogType,date,description,editorsPick,hashtags,naturalId,trendingHashtags,image,slides,title,type,uri&sort=score&start=%d&startdate=&withentity=false' % (quote_plus(keyword), curPage*100))
            d = json.loads(response.read().decode('utf-8'))
            if not 'contentList' in d: break
            for l in d['contentList']:
                if not 'uri' in l: continue
                href = l['uri']
                if not href.startswith('http'): continue
                #print(href)
                article_list.append(href)
                if outputFile: outputFile.write(href + "\n")
                if maxNum > 0 and len(article_list) >= maxNum: break
            if maxNum > 0 and len(article_list) >= maxNum: break
            if outputFile: outputFile.flush()
            curPage += 1
            self.wait()
        if outputFile: outputFile.close()
        return article_list


allClass = [NYTCollector, CNNCollector, WPCollector, ForbesCollector, WSJCollector]

if __name__ == "__main__":
    import sys
    import os

    time_range_list = ["since1851", "24hours", "7days", "30days", "12months"]
    result_type_list = ["all", "article", "blogpost", "topic", "multimedia"]
    site_type_list = ['All', 'New York Times', 'CNN', 'Washington Post', 'Forbes', 'WSJ']
    site_class = allClass

    if (len(sys.argv) > 3):
        term_1 = str(sys.argv[1])
        term_2 = str(sys.argv[2])
        site = int(sys.argv[3])
        num = int(sys.argv[4])
        _time = int(sys.argv[5]) if len(sys.argv) > 5 else 0
        _type = int(sys.argv[6]) if len(sys.argv) > 6 else 0
        term = term_1+" "+term_2

        if not os.path.exists("data/url"): os.makedirs("data/url")
        if site == 0:
            for n, cls in enumerate(site_class):
                print("Collecting article related to '%s' from %s" % (term_1 + " AND " +term_2, site_type_list[n+1]))
                with cls(wait_min = 5, wait_max = 10) as inst:
                    urls = inst.collectByKeyword(term, "data/url/%s_%s.txt" % (term_1 + "_" +term_2, site_type_list[n+1]), maxNum = num, time=time_range_list[_time], resultType=result_type_list[_type])
                    print("%d urls are collected." % len(urls))
        else:
            print("Collecting article related '%s' from %s" % (term_1 + " AND " +term_2, site_type_list[site]))
            with site_class[site-1](wait_min=5, wait_max=10) as inst:
                urls = inst.collectByKeyword(term, "data/url/%s_%s.txt" % (term_1 + "_" +term_2, site_type_list[site]), maxNum = num,
                                  time=time_range_list[_time], resultType=result_type_list[_type])
                print("%d urls are collected." % len(urls))
    else:
        print(
            "============================================="+
            "\n==== Extract Url From Online Newspaper =====\n\nPlease try again with 3~5 arguments :(1)Keywords (2)Newspaper Site (3)Number of articles [(4)time_range (5)Result_type]")
        print("Ex> extract_url.py Informatics apple 0 100 0 0\n")
        print("To crawl all articles, set 'Number of articles' to -1")

        print("-----------------Site Type -----------------")
        for i, each in enumerate(site_type_list):
            print(each + " : " + str(i))
        print("=========================================\n")
        print("-----------------Time range -----------------")
        for i, each in enumerate(time_range_list):
            print(each + " : " + str(i))
        print("=========================================\n")
        print("-----------------Result type-----------------")
        for i, each in enumerate(result_type_list):
            print(each + " : " + str(i))
        print("=========================================\n")



import gzip
import math
import os
import pickle
import sys
import numpy as np
import logging
from treform.utility.char_one_hot_vector import CharOneHotVector
from treform.utility.log_util import LogUtil

log = LogUtil.get_logger(source_filepath=None, level=logging.INFO, console_mode=True)

class DataSet(object):
    def __init__(self, features: np.ndarray = None, labels: np.ndarray = None, features_vector: CharOneHotVector = None,
                 labels_vector: CharOneHotVector = None, size=0, name: str = '', shuffle=True):
        """
        :param features: list of data
        :param labels: list of one hot vector
        """

        self.name = name
        self.size = size
        self.features_vector = features_vector
        self.labels_vector = labels_vector

        if features is None:
            self.features = np.array([])
        else:
            self.features = features if type(features) is np.ndarray else np.array(features)

        if labels is None:
            self.labels = np.array([])
        else:
            self.labels = labels if type(labels) is np.ndarray else np.array(labels)

        if features is not None and labels is not None:
            if len(features) == len(labels):
                self.size = len(features)
            else:  # invalid data size
                self.size = 0
                self.features = np.array([])
                self.labels = np.array([])

        if shuffle:
            shuffle_indices = np.random.permutation(np.arange(self.size))
            self.features = self.features[shuffle_indices]
            self.labels = self.labels[shuffle_indices]

    def next_batch(self, batch_size=50, to_one_hot_vector=True, verbose=False):
        if len(self.features) <= batch_size:
            splits = 1
        else:
            splits = len(self.features) // batch_size
            if len(self.features) % batch_size > 0:
                splits += 1

        if splits == 1:
            # log.info('next_batch(batch_size=*splits)= %s * %s = %s' % (NumUtil.comma_str(batch_size), NumUtil.comma_str(splits), NumUtil.comma_str(
            #     len(self.features))))
            if to_one_hot_vector:
                yield self.__to_one_hot_vector(self.features, self.labels, verbose=verbose)
            else:
                yield self.features, self.labels
        else:
            # log.info('next_batch(batch_size=*splits)= %s * %s = %s' % (NumUtil.comma_str(batch_size), NumUtil.comma_str(splits), NumUtil.comma_str(
            #     len(self.features))))

            for features_batch, labels_batch in zip(np.array_split(self.features, splits),
                                                    np.array_split(self.labels, splits)):
                if to_one_hot_vector:
                    features_batch, labels_batch = self.__to_one_hot_vector(features_batch, labels_batch,
                                                                            verbose=verbose)

                yield features_batch, labels_batch

    def convert_to_one_hot_vector(self, verbose=False):  # pipeline (from read to write) or tensorflow queue
        self.features, self.labels = self.__to_one_hot_vector(self.features, self.labels, verbose=verbose)
        return self

    def __repr__(self):
        return '%s "%s" (size: %s, feature: %s * %s, label: %s * %s)' % (self.__class__.__name__, self.name, self.size,
                                                                         self.features.dtype, self.features.shape,
                                                                         self.labels.dtype, self.labels.shape)

    def __str__(self):
        return self.__repr__()

    def __len__(self):
        return self.size

    def __to_one_hot_vector(self, features_batch: np.ndarray, labels_batch: np.ndarray, verbose=False):
        _features, _labels = [], []
        check_interval = min(1000, math.ceil(features_batch.shape[0]))
        for i, (feature_string, label_string) in enumerate(zip(features_batch, labels_batch)):
            if isinstance(feature_string, str) or isinstance(feature_string, list):
                feature_v = self.features_vector.to_vectors(feature_string)  # to 2 dim
                feature = np.concatenate(feature_v)  # to 1 dim
            else:
                feature = self.features_vector.to_vector(feature_string)  # to 1 dim

            if isinstance(label_string, str) or isinstance(label_string, list):
                label_v = self.labels_vector.to_vectors(label_string)  # to 2 dim
                label = np.concatenate(label_v)  # to 1 dim
            else:
                label = self.labels_vector.to_vector(label_string)  # to 1 dim

            _features.append(feature)
            _labels.append(label)

            if verbose and i % check_interval == 0:
                log.info('[%s] to_one_hot_vector %s -> %s, %s (len=%s) %s (len=%s)' % (
                i, feature_string, label, feature, len(feature), label, len(label)))
        return np.asarray(_features, dtype=np.int32), np.asarray(_labels, dtype=np.int32)

    @classmethod
    def load(cls, filepath: str, gzip_format=False, max_len=0, verbose=False):
        filename = os.path.basename(filepath)
        if gzip_format:
            f = gzip.open(filepath, 'rb')
        else:
            f = open(filepath, 'rb')

        with f:
            d = DataSet()
            d.name, d.size, d.features_vector, d.labels_vector = pickle.load(f), pickle.load(f), pickle.load(
                f), pickle.load(f)

            check_interval = min(100000, math.ceil(d.size))
            features, labels = [], []
            for i in range(d.size):
                if 0 < max_len <= len(features):
                    break
                feature, label = pickle.load(f), pickle.load(f)
                # print('load feature:', feature, 'label:', label)
                features.append(feature)
                labels.append(label)
                if verbose and i % check_interval == 0:
                    log.info('%s %.1f%% loaded.' % (filename, i / d.size * 100))
            log.info('%s 100%% loaded.' % filename)
            d.features = np.asarray(features)
            d.labels = np.asarray(labels)
            log.info('%s features shape: %s' % (filename, d.features.shape))
            log.info('%s labels shape: %s' % (filename, d.features.shape))
        return d

    def save(self, filepath: str, gzip_format=False, verbose=False):
        filename = os.path.basename(filepath)
        if gzip_format:
            f = gzip.open(filepath, 'wb')
        else:
            f = open(filepath, 'wb')

        with f:
            for o in [self.name, self.size, self.features_vector, self.labels_vector]:
                pickle.dump(o, f)

            check_interval = min(100000, math.ceil(self.size))
            for i, (feature, label) in enumerate(zip(self.features, self.labels)):
                # print('save feature:', feature, 'label:', label)
                pickle.dump(feature, f)
                pickle.dump(label, f)
                if verbose and i % check_interval == 0:
                    log.info('%s %.1f%% saved.' % (filename, i / self.size * 100))

            log.info('%s 100%% saved.' % filename)
            log.info('shape: %s' % self.features.shape)


if __name__ == '__main__':
    pass



from soynlp import DoublespaceLineCorpus
from soynlp.word import WordExtractor
from soynlp.tokenizer import LTokenizer
from soynlp.vectorizer import sent_to_word_contexts_matrix
from soynlp.word import pmi as pmi_func

class BasePMICalculator:
    INPUT=[str]
    OUTPUT=[list,tuple]

class PMICalculator(BasePMICalculator):
    def __init__(self, corpus=None):
        word_extractor = WordExtractor()
        word_extractor.train(corpus)
        cohesions = word_extractor.all_cohesion_scores()
        l_cohesions = {word: score[0] for word, score in cohesions.items()}
        tokenizer = LTokenizer(l_cohesions)
        x, self.idx2vocab = sent_to_word_contexts_matrix(
            corpus,
            windows=3,
            min_tf=10,
            tokenizer=tokenizer,  # (default) lambda x:x.split(),
            dynamic_weight=False,
            verbose=True)

        self.vocab2idx = {vocab: idx for idx, vocab in enumerate(self.idx2vocab)}

        self.pmi, px, py = pmi_func(
            x,
            min_pmi=0,
            alpha=0.0,
            beta=0.75
        )
    def __call__(self, *args, **kwargs):
        query = self.vocab2idx[args[0]]
        submatrix = self.pmi[query, :].tocsr()  # get the row of query
        contexts = submatrix.nonzero()[1]  # nonzero() return (rows, columns)
        pmi_i = submatrix.data

        most_relateds = [(idx, pmi_ij) for idx, pmi_ij in zip(contexts, pmi_i)]
        most_relateds = sorted(most_relateds, key=lambda x: -x[1])[:10]
        most_relateds = [(self.idx2vocab[idx], pmi_ij) for idx, pmi_ij in most_relateds]

        return most_relateds


import platform
from copy import deepcopy

from datetime import datetime, timedelta
from dateutil import parser

import pandas as pd
import matplotlib.font_manager as fm
import numpy as np
import matplotlib.pyplot as plt

import time
from collections import defaultdict

from matplotlib.legend_handler import HandlerLine2D
from sklearn.feature_extraction.text import CountVectorizer

# Machine learning
import sklearn
from sklearn.metrics import mean_squared_error, r2_score
from sklearn.model_selection import train_test_split
from sklearn.model_selection import KFold
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import roc_curve, auc

import tomotopy as tm

import pickle
import os

ngram_length = 3
min_yearly_df = 5

# 하이퍼 파라미터 지정.
long_ma_length = 4
short_ma_length = 2
signal_line_ma = 2
significance_ma_length = 2

significance_threshold = 0.000001
years_above_significance = 3
testing_period = 3

# Detection threshold is set such that the top 500 terms are chosen
burstiness_threshold_prediction = 0.003
burstiness_threshold_detection = 0.002451

plt.rcParams['xtick.labelsize'] = 11
plt.rcParams['ytick.labelsize'] = 11
plt.rc('font', family='sans-serif')

year_range = list(range(1988,2020))

def calc_macd(dataset):
    long_ma = dataset.ewm(span=long_ma_length).mean()
    short_ma = dataset.ewm(span=short_ma_length).mean()
    significance_ma = dataset.ewm(span=significance_ma_length).mean()
    macd = short_ma - long_ma
    signal = macd.ewm(span=signal_line_ma).mean()
    hist = macd - signal
    return long_ma, short_ma, significance_ma, macd, signal, hist

def calc_significance(stacked_vectors, significance_threshold, n):
    # Must have been above the significance threshold for two consecutive timesteps
    a = stacked_vectors>significance_threshold
    b = a.rolling(window=n).sum()
    return stacked_vectors[stacked_vectors.axes[1][np.where(b.max() >= n)[0]]]

def calc_burstiness(hist, scaling_factor):
    return hist.iloc[long_ma_length - 1:] / scaling_factor

def calc_scaling(significance_ma, method):
    if method == "max":
        scaling = significance_ma.iloc[significance_ma_length - 1:].max()
    elif method == "mean":
        scaling = significance_ma.iloc[significance_ma_length - 1:].mean()
    elif method == "sqrt":
        scaling = np.sqrt(significance_ma.iloc[significance_ma_length - 1:].max())
    return scaling

def max_burstiness(burstiness, absolute=False):
    if absolute:
        b = pd.concat([np.abs(burstiness).max(), burstiness.idxmax()], axis=1)
    else:
        b = pd.concat([burstiness.max(), burstiness.idxmax()], axis=1)
    b.columns = ["max", "location"]
    return b


def feature_selection(dataset):
    '''
    Compile the features for the prediction step
    '''
    long_ma = dataset.ewm(span=long_ma_length).mean()
    short_ma = dataset.ewm(span=short_ma_length).mean()
    significance_ma = dataset.ewm(span=significance_ma_length).mean()
    macd = short_ma - long_ma
    signal = macd.ewm(span=signal_line_ma).mean()
    hist = macd - signal

    scaling_factor = calc_scaling(significance_ma, "sqrt")
    burstiness_over_time = calc_burstiness(hist, scaling_factor)
    print('burst over time ' + str(len(burstiness_over_time)))

    burstiness = max_burstiness(burstiness_over_time)

    X = long_ma.iloc[long_ma_length:].T
    scaled_hist = hist.iloc[long_ma_length:] / scaling_factor
    scaled_signal = signal.iloc[long_ma_length:] / scaling_factor

    Xtra = pd.concat([significance_ma.iloc[-1],
                      dataset.iloc[-1],
                      significance_ma.iloc[significance_ma_length:].std() / scaling_factor,
                      significance_ma.iloc[significance_ma_length:].max(),
                      significance_ma.iloc[significance_ma_length:].min(),
                      scaling_factor
                      ], axis=1)

    pd.options.display.max_columns = None
    XY = pd.concat([X, scaled_hist.T, scaled_signal.T, Xtra], axis=1)

    print(scaled_hist.T.info(10))
    print('++++++++++++++++++')
    print(scaled_signal.T.info(10))
    print('++++++++++++++++++')
    print(Xtra.info(10))
    print('++++++++++++++++++')
    print(X.info(10))
    print('=======================')
    print(XY.info(10))

    XY.columns = [str(i) for i in range(2)] + ["hist" + str(i) for i in range(2)] + ["signal" + str(i) for i in
                                                                                    range(2)] + [
                    "significance",
                    "prevalence",
                    "scaled std",
                    "max",
                    "min",
                    "scaling"
                ]

    return XY

def balanced_subsample(x,y,subsample_size=1.0):
    # from https://stackoverflow.com/a/23479973
    class_xs = []
    min_elems = None

    for yi in np.unique(y):
        elems = x[(y == yi)]
        class_xs.append((yi, elems))
        if min_elems == None or elems.shape[0] < min_elems:
            min_elems = elems.shape[0]

    use_elems = min_elems
    if subsample_size < 1:
        use_elems = int(min_elems*subsample_size)

    xs = []
    ys = []

    for ci,this_xs in class_xs:
        if len(this_xs) > use_elems:
            np.random.shuffle(this_xs)

        x_ = this_xs[:use_elems]
        y_ = np.empty(use_elems)
        y_.fill(ci)

        xs.append(x_)
        ys.append(y_)

    xs = np.concatenate(xs)
    ys = np.concatenate(ys)

    return xs,ys


def get_prevalence(cluster, bursts, burstvectors, unique_time_stamp):
    indices = []
    for term in cluster:
        if term in bursts:
            indices.append(bursts.index(term))

    prevalence = []
    for year in unique_time_stamp:
        prevalence.append(
            100 * np.sum(np.sum(burstvectors[year][:, indices], axis=1) > 0) / burstvectors[year].shape[0])

    return prevalence

def compute_term_burstiness(dataset):
    # Build a vocabulary
    # We have to build a vocabulary before we vectorise the data. This is because we want to set limits on the size of the vocabulary.

    vocab = set()

    df = pd.read_csv(dataset)

    grouped_df = df.groupby(df.columns[0])

    for key, item in grouped_df:
        a_group = grouped_df.get_group(key)

        print(a_group.head(10))
        # The same as above, applied year by year instead.
        t0 = time.time()

        vectorizer = CountVectorizer(min_df=min_yearly_df)
        print(a_group.iloc[:, 4].head(10))

        vector = vectorizer.fit_transform(a_group.iloc[:, 4])

        # Save the new words
        vocab = vocab.union(vectorizer.vocabulary_.keys())
        time_stamp = a_group.iloc[:, 0].tolist()[0]
        print(time_stamp, len(vocab), time.time() - t0)

    vocabulary = {}
    i = 0
    for v in vocab:
        vocabulary[v] = i
        i += 1

    print(len(vocabulary.keys()))

    # Go year by year and vectorise based on our vocabulary
    # We read in the cleaned data and vectorise it according to our vocabulary.
    vectors = []
    grouped_df = df.groupby(df.columns[0])

    for key, item in grouped_df:
        a_group = grouped_df.get_group(key)

        # The same as above, applied year by year instead.
        t0 = time.time()

        vectorizer = CountVectorizer(vocabulary=vocabulary)

        vectors.append(vectorizer.fit_transform(a_group.iloc[:, 4]))
        time_stamp = a_group.iloc[:, 0].tolist()[0]
        print(time_stamp, time.time() - t0)

    # Summing the vectors
    # We sum the vectors along columns, so that we have the popularity of each term in each year.

    summed_vectors = []
    for y in range(len(vectors)):
        vector = vectors[y]

        # Set all elements that are greater than one to one -- we do not care if a word is used multiple times in
        # the same document
        vector[vector > 1] = 1

        # Sum the vector along columns
        summed = np.squeeze(np.asarray(np.sum(vector, axis=0)))

        # Normalise by dividing by the number of documents in that year
        normalised = summed / vector.shape[0]

        # Save the summed vector
        summed_vectors.append(normalised)

    # Stack vectors vertically, so that we have the full history of popularity/time for each term
    stacked_vectors = np.stack(summed_vectors, axis=1)

    print(stacked_vectors.shape)

    stacked_vectors = pd.DataFrame(stacked_vectors.transpose(), columns=list(vocabulary.keys()))

    normalisation = stacked_vectors.sum(axis=1)
    stacked_vectors = stacked_vectors.divide(normalisation, axis='index') * 100

    stacked_vectors = calc_significance(stacked_vectors, significance_threshold, years_above_significance)
    print(stacked_vectors.shape)

    # Calculate burstiness
    long_ma, short_ma, significance_ma, macd, signal, hist = calc_macd(stacked_vectors)
    scaling_factor = calc_scaling(significance_ma, "sqrt")
    burstiness_over_time = calc_burstiness(hist, scaling_factor)
    burstiness = max_burstiness(burstiness_over_time)

    # Set a threshold such that the top 500 bursty terms are included
    print(np.sum(burstiness["max"] > 0.002451))

    bursts = list(burstiness["max"].index[np.where(burstiness["max"] > burstiness_threshold_detection)[0]])
    print(bursts)

    # Cluster bursts based on co-occurence
    # vectorise again, using these terms only
    burstvectors = {}
    grouped_df = df.groupby(df.columns[0])

    corpus = tm.utils.Corpus()

    unique_time_stamp = []
    for key, item in grouped_df:
        a_group = grouped_df.get_group(key)
        # The same as above, applied year by year instead.
        t0 = time.time()

        vectorizer = CountVectorizer(vocabulary=bursts)
        vector = vectorizer.fit_transform(a_group.iloc[:, 4])

        for i, _doc in enumerate(a_group.iloc[:, 4].tolist()):
            corpus.add_doc(_doc.strip().split())
            if i % 10 == 0:
                print('Document #{} has been loaded'.format(i))

        # If any element is larger than one, set it to one
        vector.data = np.where(vector.data > 0, 1, 0)
        time_stamp = a_group.iloc[:, 0].tolist()[0]
        unique_time_stamp.append(time_stamp)
        burstvectors[time_stamp] = vector

        print(time_stamp, time.time() - t0)

    with open('../models/unique_time_stamp.pickle', 'wb') as handle:
        pickle.dump(unique_time_stamp, handle)

    clusters = defaultdict(list)

    model = tm.LDAModel(k=30, min_cf=10, min_df=5, rm_top=50, corpus=corpus)
    model.optim_interval = 20
    model.burn_in = 200
    model.train(0)

    # Let's train the model
    for i in range(0, 1500, 20):
        print('Iteration: {:04} LL per word: {:.4}'.format(i, model.ll_per_word))
        model.train(20)
    print('Iteration: {:04} LL per word: {:.4}'.format(1000, model.ll_per_word))

    # extract candidates for auto topic labeling
    extractor = tm.label.PMIExtractor(min_cf=10, min_df=10, max_len=5, max_cand=10000, normalized=True)
    cands = extractor.extract(model)
    labeler = tm.label.FoRelevance(model, cands, min_df=5, smoothing=1e-2, mu=0.25)

    cluster_label = {}
    for k in range(model.k):
        print('Topic #{}'.format(k))
        m = 0
        for label, score in labeler.get_topic_labels(k, top_n=1):
            if m > 0:
                continue

            print("Labels:", label)
            cluster_label[k] = label
            m += 1

        for word, prob in model.get_topic_words(k, top_n=20):
            print('\t', word, prob, sep='\t')
            clusters[k].append(word)

    for key in sorted(clusters.keys()):
        print(key, ':', ', '.join(clusters[key]))

    font_path = ''
    if platform.system() == 'Windows':
        # Window의 경우 폰트 경로
        font_path = 'C:/Windows/Fonts/malgun.ttf'
    elif platform.system() == 'Darwin':
        # for Mac
        font_path = '/Library/Fonts/AppleGothic.ttf'

    font_name = fm.FontProperties(fname=font_path).get_name()
    plt.rc('font', family=font_name)
    plt.rc('axes', unicode_minus=False)

    total = model.k
    y_val = int(np.ceil(total / 4))

    # Graph selected bursty terms over time
    # yplots = 13
    # xplots = 4
    yplots = y_val
    xplots = 4
    fig, axs = plt.subplots(yplots, xplots)
    plt.subplots_adjust(right=1, hspace=0.9, wspace=0.3)
    plt.suptitle('Prevalence of selected bursty clusters over time', fontsize=14)
    fig.subplots_adjust(top=0.95)
    fig.set_figheight(16)
    fig.set_figwidth(12)

    print(bursts[0])
    print(burstvectors[0])

    prevalences = []
    for i, cluster in enumerate(clusters):
        prevalence = get_prevalence(clusters[cluster], bursts, burstvectors, unique_time_stamp)

        x = range(0, len(prevalence))

        prevalences.append(prevalence)
        title = cluster_label[cluster]
        axs[int(np.floor((i / xplots) % yplots)), i % xplots].plot(x, prevalence, color='k', ls='-', label=title)
        axs[int(np.floor((i / xplots) % yplots)), i % xplots].grid()
        ymax = np.ceil(max(prevalence) * 2) / 2
        if ymax == 0.5 and max(prevalence) < 0.25:
            ymax = 0.25
        elif ymax == 2.5:
            ymax = 3
        axs[int(np.floor((i / xplots) % yplots)), i % xplots].set_ylim(0, ymax)
        axs[int(np.floor((i / xplots) % yplots)), i % xplots].set_xlim(0, len(prevalence))
        axs[int(np.floor((i / xplots) % yplots)), i % xplots].set_title(title, fontsize=12)

        if i % yplots != yplots - 1:
            axs[i % yplots, int(np.floor((i / yplots) % xplots))].set_xticklabels([])
        else:
            axs[i % yplots, int(np.floor((i / yplots) % xplots))].set_xticklabels([1988, 1998, 2008, 2018])

    axs[6, 0].set_ylabel('Percentage of documents containing term (%)', fontsize=12)

    # plt.show()
    plt.savefig('burst_example.png')
    plt.close(fig)

    with open('../models/burstiness_stacked_vec.pickle', 'wb') as handle:
        pickle.dump(stacked_vectors, handle)

#Build the development set
def build_development_set(stacked_vectors, unique_time_stamp, cutoff):
    reversed_stamp = deepcopy(unique_time_stamp)
    reversed_stamp.sort(reverse=True)

    print(reversed_stamp)
    recent = reversed_stamp[:10]
    print('--------------------------')
    print(recent)

    development_data = {}
    for _time_period in range(recent[len(recent)-1], recent[0]):
        _time_period_idx = unique_time_stamp.index(_time_period)

        # Use our three-year method to calc significance
        valid_vectors = calc_significance(stacked_vectors[:_time_period + 1], significance_threshold, 3)

        # Recalculate the macd things based on this more limited dataset
        long_ma, short_ma, significance_ma, macd, signal, hist = calc_macd(valid_vectors)

        #needs to be flexible for other types of timestamp - days, hours, minutes, etc
        # Calculate scaling factor
        scaling_factor = calc_scaling(significance_ma.iloc[max(long_ma_length, _time_period_idx - 13):_time_period_idx + 1], "sqrt")

        # Calculate the burstiness
        burstiness_over_time = calc_burstiness(hist, scaling_factor)
        burstiness = max_burstiness(burstiness_over_time)

        # Choose terms that are above both thresholds (burstiness, and also most recent year was significant)
        burst_idx = np.where((burstiness["max"] > 0.0012) & (significance_ma.iloc[_time_period_idx] > significance_threshold))[0]

        # Find the actual names of these terms
        bursts = valid_vectors.keys()[burst_idx]

        #needs to revist this logic -- minus 19! : update - 50 days?
        # Create a new, much smaller dataset

        dataset = stacked_vectors[bursts].iloc[_time_period_idx - 13:_time_period_idx + 1]

        # needs to set the threshold not just like 2015
        # Get the scaled y values
        if _time_period < cutoff:
            y = stacked_vectors[bursts].iloc[_time_period_idx + testing_period]

        # Select features and store the data
        development_data[_time_period] = {}
        development_data[_time_period]["X"] = feature_selection(dataset)
        if _time_period < cutoff:
            development_data[_time_period]["y"] = y - development_data[_time_period]["X"]["significance"]
        print(_time_period, len(bursts))

    print(len(development_data))
    return development_data

#Choosing a max depth for the random forest
def choose_max_depth(development_data, unique_time_stamp, cutoff):
    reversed_stamp = deepcopy(unique_time_stamp)
    reversed_stamp.sort(reverse=True)

    #
    # recent[len(recent)-3] is cutoff value
    #print(development_data)
    recent = reversed_stamp[:10]
    print(recent)

    X_array = []
    for year in range(recent[len(recent) - 3], cutoff):
        if development_data[year] is not None:
            X_array.append(development_data[year]["X"])
            print("FOUND!!!!!!!!!!!!!!!!!!!")

    if len(X_array) > 0:
        X = np.array(pd.concat(X_array))


    y_array = []
    for year in range(recent[len(recent) - 3], cutoff):
        if development_data[year] is not None:
            y_array.append(development_data[year]["y"])
            print("FOUND!!!!!!!!!!!!!!!!!!!")

    if len(y_array) > 0:
        y = np.array(pd.concat(y_array))
    else:
        return

    #X = np.array(pd.concat([development_data[year]["X"] for year in range(recent[len(recent)-3], cutoff)]))
    #y = np.array(pd.concat([development_data[year]["y"] for year in range(recent[len(recent)-3], cutoff)]))
    y_thresh = np.zeros_like(y)
    y_thresh[y > 0] = 1

    # Balance the sample
    X, y_thresh = balanced_subsample(X, y_thresh, subsample_size=1.0)
    x_train, x_test, y_train, y_test = train_test_split(X, y_thresh, test_size=0.33, random_state=1)

    max_depths = np.linspace(1, 32, 32, endpoint=True)
    train_results = []
    test_results = []
    for max_depth in max_depths:
        rf = RandomForestClassifier(n_estimators=100, max_depth=max_depth)
        rf.fit(x_train, y_train)
        train_pred = rf.predict(x_train)
        false_positive_rate, true_positive_rate, thresholds = roc_curve(y_train, train_pred)
        roc_auc = auc(false_positive_rate, true_positive_rate)
        train_results.append(roc_auc)
        y_pred = rf.predict(x_test)
        false_positive_rate, true_positive_rate, thresholds = roc_curve(y_test, y_pred)
        roc_auc = auc(false_positive_rate, true_positive_rate)
        test_results.append(roc_auc)

    line1, = plt.plot(max_depths, train_results, 'b', label="Train AUC")
    line2, = plt.plot(max_depths, test_results, 'r', label="Test AUC")
    plt.legend(handler_map={line1: HandlerLine2D(numpoints=2)})
    plt.ylabel('AUC score')
    plt.xlabel('Tree depth')
    plt.title('Effect of changing the maximum depth of the random forest on classifier accuracy')
    plt.show()

#Choosing the number of estimators
def choose_estimators(development_data, unique_time_stamp):
    reversed_stamp = deepcopy(unique_time_stamp)
    reversed_stamp.sort(reverse=True)
    #
    # recent[len(recent)-3] is cutoff value
    #
    print(development_data)
    recent = reversed_stamp[:10]
    X = np.array(pd.concat([development_data[year]["X"] for year in range(recent[len(recent)-3], recent[0])]))
    y = np.array(pd.concat([development_data[year]["y"] for year in range(recent[len(recent)-3],recent[0])]))

    #X = np.array(pd.concat([development_data[year]["X"] for year in range(2008, 2015)]))
    #y = np.array(pd.concat([development_data[year]["y"] for year in range(2008, 2015)]))
    y_thresh = np.zeros_like(y)
    y_thresh[y > 0] = 1

    # Balance the sample
    X, y_thresh = balanced_subsample(X, y_thresh, subsample_size=1.0)
    x_train, x_test, y_train, y_test = train_test_split(X, y_thresh, test_size=0.33, random_state=3)

    n_estimators = [1, 2, 4, 8, 16, 32, 64, 100, 150, 200, 300, 500, 800]
    train_results = []
    test_results = []
    for estimator in n_estimators:
        rf = RandomForestClassifier(n_estimators=estimator, max_depth=13, n_jobs=-1)
        rf.fit(x_train, y_train)
        train_pred = rf.predict(x_train)
        false_positive_rate, true_positive_rate, thresholds = roc_curve(y_train, train_pred)
        roc_auc = auc(false_positive_rate, true_positive_rate)
        train_results.append(roc_auc)
        y_pred = rf.predict(x_test)
        false_positive_rate, true_positive_rate, thresholds = roc_curve(y_test, y_pred)
        roc_auc = auc(false_positive_rate, true_positive_rate)
        test_results.append(roc_auc)
    from matplotlib.legend_handler import HandlerLine2D
    line1, = plt.plot(n_estimators, train_results, 'b', label='Train AUC')
    line2, = plt.plot(n_estimators, test_results, 'r', label='Test AUC')
    plt.legend(handler_map={line1: HandlerLine2D(numpoints=2)})
    plt.ylabel('AUC score')
    plt.xlabel('n_estimators')
    plt.title('Effect of changing the number of estimators of the random forest on classifier accuracy')
    plt.show()

#The effect of changing the burstiness threshold and prediction interval on the random forest classifier
def change_bustiness_threshold(stacked_vectors, development_data):
    scores = {}
    for threshold in np.arange(0.0006, 0.0017, 0.0002):
        scores[threshold] = {}
        for year in range(2008, 2013):
            year_idx = year_range.index(year)

            # Use our three-year method to calc significance
            valid_vectors = calc_significance(stacked_vectors[:year_idx + 1], significance_threshold, 3)

            # Recalculate the macd things based on this more limited dataset
            long_ma, short_ma, significance_ma, macd, signal, hist = calc_macd(valid_vectors)

            # Calculate scaling factor
            scaling_factor = calc_scaling(significance_ma.iloc[max(long_ma_length, year_idx - 19):year_idx + 1], "sqrt")

            # Calculate the burstiness
            burstiness_over_time = calc_burstiness(hist, scaling_factor)
            burstiness = max_burstiness(burstiness_over_time)

            # Choose terms that are above both thresholds (burstiness, and also most recent year was significant)
            burst_idx = \
            np.where((burstiness["max"] > threshold) & (significance_ma.iloc[year_idx] > significance_threshold))[0]

            # Find the actual names of these terms
            bursts = valid_vectors.keys()[burst_idx]

            # Create a new, much smaller dataset
            dataset = stacked_vectors[bursts].iloc[year_idx - 19:year_idx + 1]

            # Select features and store the data
            development_data[year] = {}
            development_data[year]["X"] = feature_selection(dataset)

            development_data[year]["y"] = {}

            for interval in range(1, 6):
                # Get the scaled y values
                y = stacked_vectors[bursts].iloc[year_idx + interval]
                development_data[year]["y"][interval] = y - development_data[year]["X"]['significance']

        X = np.array(pd.concat([development_data[year]["X"] for year in range(2008, 2013)]))

        for interval in range(1, 6):
            scores[threshold][interval] = {}
            scores[threshold][interval]['scores'] = []
            y = np.array(pd.concat([development_data[year]["y"][interval] for year in range(2008, 2013)]))

            # Binarise y data
            y_thresh = np.zeros_like(y)
            y_thresh[y > 0] = 1

            # Balance the sample
            X_bal, y_thresh = balanced_subsample(X, y_thresh, subsample_size=1.0)

            scores[threshold][interval]['size'] = len(y_thresh)
            kf = KFold(n_splits=10, shuffle=True)
            for train, test in kf.split(X_bal):
                clf = RandomForestClassifier(n_estimators=150, max_depth=13)

                clf.fit(X_bal[train], y_thresh[train])
                preds = clf.predict(X_bal[test])

                new_scores = [
                    sklearn.metrics.accuracy_score(y_thresh[test], preds),
                    sklearn.metrics.f1_score(y_thresh[test], preds),
                    np.sum(y_thresh[test] == 0) / len(y_thresh[test])
                ]
                scores[threshold][interval]['scores'].append(new_scores)

            print(threshold, interval, len(y_thresh),
                  np.round(np.mean(np.array(scores[threshold][interval]['scores'])[:, 0]), 3)
                  )

#Format for table
def format_table(scores):
    for threshold in np.arange(0.0006, 0.0017, 0.0002):
        print(threshold, '&',
              scores[threshold][3]['size'], '&',
              np.round(np.mean(np.array(scores[threshold][3]['scores'])[:, 0]), 2),
              '$\pm$',
              np.round(np.std(np.array(scores[threshold][3]['scores'])[:, 0]), 2), '&',
              np.round(np.mean(np.array(scores[threshold][3]['scores'])[:, 1]), 2),
              '$\pm$',
              np.round(np.std(np.array(scores[threshold][3]['scores'])[:, 1]), 2),
              '\\\\'
              )
    plt.rc('font', family='sans-serif')
    plt.rc('xtick', labelsize='medium')
    plt.rc('ytick', labelsize='medium')
    line_styles = ['-', '--', ':']
    col = 0.5
    fig = plt.figure(figsize=(6, 3.7))
    ax = fig.add_subplot(1, 1, 1)
    ax.set_title('Choosing a prediction interval, I', fontsize=13)
    ax.grid()
    ax.set_ylim(0.65, 0.9)
    # ax.set_xlim(1,5)

    ax.set_ylabel('F1 score', fontsize=12)
    ax.set_xlabel('Number of years in future', fontsize=12)

    plt.xticks(range(1, 6), range(1, 6))

    thresholds = ["0.0006", "0.0008", "0.0010", "0.0012", "0.0014", "0.0016"]
    for i, threshold in enumerate(np.arange(0.0006, 0.0017, 0.0002)):
        y = []
        yerr = []
        for interval in range(1, 6):
            s = np.array(scores[threshold][interval]['scores'])
            y.append(np.mean(s[:, 1]))
            yerr.append(np.std(s[:, 1]))

        ax.errorbar(range(1, 6), y, yerr=yerr, color=str(col), label=thresholds[i], fmt='--o')

        col -= 0.1

    # ax.legend(ncol=3, mode="expand")
    ax.legend(fontsize=11)

#Train a single classifier on all the data from 1988-2014
def train_classifier(stacked_vectors, unique_time_stamp, cutoff):
    threshold = 0.0012
    interval = 3

    reversed_stamp = deepcopy(unique_time_stamp)
    reversed_stamp.sort(reverse=True)

    print(reversed_stamp)
    recent = reversed_stamp[:10]
    print('--------------------------')
    print(recent)

    development_data = {}
    for _time_period in range(recent[len(recent) - 1], recent[0]):
        _time_period_idx = unique_time_stamp.index(_time_period)
        # Use our three-year method to calc significance
        valid_vectors = calc_significance(stacked_vectors[:_time_period_idx + 1], significance_threshold, 3)

        # Recalculate the macd things based on this more limited dataset
        long_ma, short_ma, significance_ma, macd, signal, hist = calc_macd(valid_vectors)

        # Calculate scaling factor
        scaling_factor = calc_scaling(significance_ma.iloc[max(long_ma_length, _time_period_idx - 13):_time_period_idx + 1], "sqrt")

        # Calculate the burstiness
        burstiness_over_time = calc_burstiness(hist, scaling_factor)
        burstiness = max_burstiness(burstiness_over_time)

        # Choose terms that are above both thresholds (burstiness, and also most recent year was significant)
        burst_idx = \
        np.where((burstiness["max"] > threshold) & (significance_ma.iloc[_time_period_idx] > significance_threshold))[0]

        # Find the actual names of these terms
        bursts = valid_vectors.keys()[burst_idx]

        # Create a new, much smaller dataset
        dataset = stacked_vectors[bursts].iloc[_time_period_idx - 13:_time_period_idx + 1]

        # Select features and store the data
        development_data[_time_period] = {}
        development_data[_time_period]["X"] = feature_selection(dataset)
        if _time_period < cutoff:
            y = stacked_vectors[bursts].iloc[_time_period + interval]
            development_data[_time_period]["y"] = y - development_data[_time_period]["X"]['significance']

    recent = reversed_stamp[:10]
    print(recent)

    X_array = []
    for year in range(recent[len(recent) - 3], cutoff):
        if development_data[year] is not None:
            X_array.append(development_data[year]["X"])
            print("FOUND!!!!!!!!!!!!!!!!!!!")

    if len(X_array) > 0:
        X = np.array(pd.concat(X_array))

    y_array = []
    for year in range(recent[len(recent) - 3], cutoff):
        if development_data[year] is not None:
            y_array.append(development_data[year]["y"])
            print("FOUND!!!!!!!!!!!!!!!!!!!")

    if len(y_array) > 0:
        y = np.array(pd.concat(y_array))
    else:
        return

    #X = np.array(pd.concat([development_data[year]["X"] for year in range(2008,2015)]))
    #y = np.array(pd.concat([development_data[year]["y"] for year in range(2008,2015)]))

    # Binarise y data
    y_thresh = np.zeros_like(y)
    y_thresh[y>0] = 1

    # Balance the sample
    X_bal, y_thresh = balanced_subsample(X, y_thresh,subsample_size=1.0)

    clf = RandomForestClassifier(n_estimators=150, max_depth=13)
    clf.fit(X_bal, y_thresh)

    return clf

def predict(burstiness, significance_ma, valid_vectors, stacked_vectors, year_idx, clf):
    # Choose terms that are above both thresholds
    burst_idx = np.where(
        (burstiness["max"] > burstiness_threshold_prediction) & (significance_ma.iloc[29] > significance_threshold))[0]

    # Find the actual names of these terms
    bursts = list(valid_vectors.keys()[burst_idx])

    # Create a new, much smaller dataset
    dataset = stacked_vectors[bursts].iloc[year_idx - 19:year_idx + 1]

    X_test = np.array(feature_selection(dataset))

    preds = clf.predict(X_test)
    rise = []
    fall = []
    for i, p in enumerate(preds):
        if p > 0.5:
            rise.append(bursts[i])
        else:
            fall.append(bursts[i])

    print(', '.join(rise))
    print()
    print(', '.join(fall))


def compute_pure_term_burstiness(documents, label_list):
    # Build a vocabulary
    # We have to build a vocabulary before we vectorise the data. This is because we want to set limits on the size of the vocabulary.

    vocab = set()

    df = pd.DataFrame([(label_list, documents) for label_list, documents in zip(label_list, documents)])
    print(df)

    grouped_df = df.groupby(df.columns[0])

    for key, item in grouped_df:
        a_group = grouped_df.get_group(key)

        print(a_group.head(10))
        # The same as above, applied year by year instead.
        t0 = time.time()

        vectorizer = CountVectorizer(min_df=min_yearly_df)
        print(a_group.iloc[:,1].head(10))

        vector = vectorizer.fit_transform(a_group.iloc[:, 1])

        # Save the new words
        vocab = vocab.union(vectorizer.vocabulary_.keys())
        time_stamp = a_group.iloc[:, 0].tolist()[0]
        print(time_stamp, len(vocab), time.time() - t0)

    vocabulary = {}
    i = 0
    for v in vocab:
        vocabulary[v] = i
        i += 1

    print(len(vocabulary.keys()))

    vocab = pd.DataFrame(vocabulary.items())[[0]]
    #vocab.to_csv('../sample_data/vocab.csv')
    # Go year by year and vectorise based on our vocabulary
    # We read in the cleaned data and vectorise it according to our vocabulary.
    vectors = []
    grouped_df = df.groupby(df.columns[0])

    for key, item in grouped_df:
        a_group = grouped_df.get_group(key)

        # The same as above, applied year by year instead.
        t0 = time.time()

        vectorizer = CountVectorizer(vocabulary=vocabulary)

        vectors.append(vectorizer.fit_transform(a_group.iloc[:, 1]))
        time_stamp = a_group.iloc[:, 0].tolist()[0]
        print(time_stamp, time.time() - t0)

    # Summing the vectors
    # We sum the vectors along columns, so that we have the popularity of each term in each year.
    summed_vectors = []
    for y in range(len(vectors)):
        vector = vectors[y]

        # Set all elements that are greater than one to one -- we do not care if a word is used multiple times in
        # the same document
        vector[vector > 1] = 1

        # Sum the vector along columns
        summed = np.squeeze(np.asarray(np.sum(vector, axis=0)))

        # Normalise by dividing by the number of documents in that year
        normalised = summed / vector.shape[0]

        # Save the summed vector
        summed_vectors.append(normalised)

    # Stack vectors vertically, so that we have the full history of popularity/time for each term
    stacked_vectors = np.stack(summed_vectors, axis=1)
    #print(stacked_vectors.shape)

    stacked_vectors = pd.DataFrame(stacked_vectors.transpose(), columns=list(vocabulary.keys()))
    #print(stacked_vectors.shape)

    #stacked_vectors.to_csv('../sample_data/stacked_vectors.csv')
    #stacked_vectors.index = list(range(20170101, 20171024))
    print(stacked_vectors.index)

    long_ma, short_ma, significance_ma, macd, signal, hist = calc_macd(stacked_vectors)
    scaling_factor = calc_scaling(significance_ma, "mean")
    #print(hist)
    burstiness_over_time = calc_burstiness(hist, scaling_factor)
    #burstiness = max_burstiness(burstiness_over_time)

    dir = "./results"
    if not os.path.exists(dir):
        os.makedirs(dir)

    burstiness_over_time.to_csv(dir + '/bursty_terms.csv')

    print(burstiness_over_time)



import urllib
import urllib.request
import urllib.parse
import bs4
import json
import re
import sys
import time, random

from treform.collector.colBasic import *

class ColBigkinds(ColBasic):
    def getInfo(params):
        return '빅카인즈 수집기', 0

    def getName(self):
        return "colBigKinds"

    def getSearchURL(self):
        return 'https://www.bigkinds.or.kr/news/detailSearch.do'

    def makeParameter(query, page, startDate, endDate):
        return {
            'pageInfo': 'newsResult',
            'login_chk': 'null',
            'LOGIN_SN': 'null',
            'LOGIN_NAME': 'null',
            'indexName': 'news',
            'keyword': query,
            'byLine': '',
            'searchScope': '1',
            'searchFtr': '1',
            'startDate': startDate,
            'endDate': endDate,
            'sortMethod': 'date',
            'contentLength': '100',
            'providerCode': '',
            'categoryCode': '',
            'incidentCode': '',
            'dateCode': '',
            'highlighting': 'true',
            'sessionUSID': '',
            'sessionUUID': 'test',
            'listMode': '',
            'categoryTab': '',
            'newsId': '',
            'filterProviderCode': '',
            'filterCategoryCode': '',
            'filterIncidentCode': '',
            'filterDateCode': '',
            'filterAnalysisCode': '',
            'startNo': page,
            'resultNumber': '100',
            'topmenuoff': '',
            'resultState': 'detailSearch',
            'keywordJson': '{"searchDetailTxt1":"' + query + '","agreeDetailTxt1":"","needDetailTxt1":"","exceptDetailTxt1":"","o_id":"option1","startDate":"' + startDate + '","endDate":"' + endDate + '","providerNm":"","categoryNm":"","incidentCategoryNm":"","providerCode":"","categoryCode":"","incidentCategoryCode":"","searchFtr":"1","searchScope":"1","searchKeyword":"' + query + '"}',
            'keywordFilterJson': '',
            'interval': '',
            'quotationKeyword1': '',
            'quotationKeyword2': '',
            'quotationKeyword3': '',
            'searchFromUseYN': 'N',
            'mainTodayPersonYn': '',
            'period': ''
        }




import math

def tf_idf(corpus, package):
	dictlist = {}
	doclen = {}
	docname = package ["docname"]
	weights = package ["weights"]

	for i in corpus:
		docl = i["document"]
		doclen[docl] = i["length"]
		for j in i["split_sentence"].split():
			# dctlist : doc —— word —— frequency
			if docl not in dictlist:
				dictlist[docl] = {}
			if j not in dictlist[docl]:
				dictlist[docl][j] = 0
			dictlist[docl][j] += 1

			# doclist :  word ——　doc set
			if j not in weights:
				weights[j] = set()
			weights[j].add(docl)
			docname.add(docl)
	for word in weights:
		weights[word] = math.log( ( 1+len(docname)*1.0)/(len(weights[word])*1.0),2)
	tf_idf_weight = {}
	for doc in dictlist:
		tf_idf_weight[doc] = {}
		for word in dictlist[doc]:
			# tf:
			tf_idf_weight[doc][word] = dictlist[doc][word]*1.0 / (doclen[doc]*1.0)
			# tf*idf
			tf_idf_weight[doc][word] *= weights[word]
	package ["docname"] = docname
	package ["weights"] = weights
	#print(tf_idf_weight)

	return tf_idf_weight


import getpass
import inspect
import os
import platform
import socket


def home_dir():
    return os.path.expanduser("~")


def real_path(filepath=None):
    """ 이 함수를 호출한 소스의 상대 경로와 filepath의 상대경로를 조합하여, 절대 경로를 생성함. (shell command 용) """
    frame = inspect.stack()[1]
    caller = inspect.getmodule(frame[0])
    if filepath:
        return os.path.realpath(os.path.join(os.path.dirname(os.path.realpath(caller.__file__)), filepath))
    else:
        return os.path.realpath(caller.__file__)


# noinspection PyUnresolvedReferences
def get_local_address():
    if not hasattr(get_local_address, 'LOCAL_ADDRESS'):
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('gmail.com', 80))
        setattr(get_local_address, 'LOCAL_ADDRESS', s.getsockname()[0])
        s.close()
    return get_local_address.LOCAL_ADDRESS


def get_username():
    """shell 로그인 계정"""
    return getpass.getuser()


def is_osx_os():
    """ Mac """
    return platform.system().lower() == 'darwin'


def is_linux_os():
    """ Linux """
    return platform.system().lower() == 'linux'


def is_windows_os():
    """ Windows """
    return platform.system().lower() == 'windows'


def is_pycharm_remote():
    return os.getenv('PYTHONUNBUFFERED') is not None


def is_my_pc():
    """ 맥 또는 내 PC로 판단함."""
    return is_osx_os() or is_windows_os()


def is_my_gpu_pc():
    """ GPU 장착된 PC """
    return hostname() == 'office'


def is_server():
    """ batch script 실행중인지 """
    return not is_my_pc() and not is_pycharm_remote()


def hostname():
    """ hostname 확인 (linux, max 정상 동작. windows 확인 필요 """
    return socket.gethostname()


def api_server_hostname():
    """ 접속 가능한 API서버의 hostname을 리턴 """
    if is_my_pc():
        return 'localhost'  # test server
    else:
        return 'api-local'  # real server


def db_hostname():
    """ DB 서버 hostname 조회 """
    if is_my_pc():
        return 'localhost'  # test server
    else:
        return 'office'  # real server


def elasticsearch_hostname():
    return 'office'  # real server


def neo4j_hostname():
    return 'office'  # real server


def redis_hostname():
    return 'office'  # real server


def hts_server_hostname():
    """ HTS(windows) hostname 조회 """
    if is_osx_os():  # I'm paralles host
        return 'vm_guest'
    elif is_windows_os():  # I'm HTS server
        return 'localhost'
    else:  # I'm ubuntu
        return 'hts-local'  # 호스트에서 port-forwarding으로 게스트 포트 열어줌. 10.211.55.4(parallels),    10.0.2.1(virtualbox)


if __name__ == '__main__':
    print(hostname())
    print(get_local_address())
    # print(platform.system().lower())
    print(real_path('logs'))


import pandas as pd
from .spelling_unicode import split_syllables


def build_decomposed_dictionary(input_filename: str, output_file: str):
    vocab = pd.read_csv(input_filename, sep=" ", names=["term", "count"])
    vocab.term = vocab.term.map(split_syllables)
    vocab.to_csv(output_file, sep=" ", header=None, index=None)


def build_decomposed_bigram_dictionary(input_filename: str, output_file: str):
    bigrams = pd.read_csv(input_filename, sep=" ", names=["term1", "term2", "count"])
    bigrams.term1 = bigrams.term1.map(split_syllables)
    bigrams.term2 = bigrams.term2.map(split_syllables)
    bigrams.to_csv(output_file, sep=" ", header=None, index=None)




from nltk.corpus import wordnet as wn
import random
import pandas as pd
import numpy as np
import gensim
import pickle
from sklearn.model_selection import train_test_split

"""
This file is used to create list of synonyms, antonyms and irrelevant word pairs
"""

def generate_synonyms(file_name='synonyms_max.txt'):
    #generate 44k pairs of synonyms and save to file_name
    f = open(file_name, 'w')
    count = 0
    for i in wn.all_synsets():
        if len(i.lemma_names()) > 1:
            """
            current_word = i.lemma_names()[0]
            if '_' in current_word:
                continue
            for word in i.lemma_names()[1:]:
                if '_' in word:
                    continue
            """
            l = i.lemma_names()
            for i in range(len(l)):
                for j in range(i+1, len(l)):
                    if '_' not in l[i] and '_' not in l[j]:
                        f.write(l[i]+', '+l[j]+'\n')
                        count+=1
    print(count)
    f.close()


def generate_antonyms(file_name='antonyms.txt'):
    # generate 3.2k pairs of antonyms and save to file_name
    antonym_list=[]
    for i in wn.all_synsets():
        for m in i.lemmas():
            if len(m.antonyms())>0:
                for j in m.antonyms():
                    if j.name()<m.name():
                        antonym_list.append((j.name(), m.name()))
                    else:
                        antonym_list.append((m.name(), j.name()))
                    print(j.name() + ", " + m.name())
    antonym_list = set(antonym_list)
    f = open(file_name, 'wb')
    count = 0
    for i in antonym_list:
        if '_' in i[0] or '_' in i[1]:
            continue
        f.write(i[0]+', '+i[1]+'\n')
        count+=1
    print(count)
    f.close()

def generate_irrelevant(file_name='irrelevent_min2.txt'):
    #generate irrelevant word pairs, will be used as negative label
    all_lemma = [i for i in wn.all_lemma_names() if '_' not in i]
    length = len(all_lemma)
    count = 0
    f = open(file_name, 'w')
    for i in wn.all_synsets():
        m = len(i.lemma_names())/3+1
        m = int(m)
        for j in range(m):
            current_word = i.lemma_names()[random.randint(0, len(i.lemma_names())-1)]
            if '_' not in current_word:
                for j in range(2):
                    word = all_lemma[random.randint(0,length-10)]
                    if word not in i.lemma_names():
                        count+=1
                        f.write(str(current_word)+', '+str(word)+'\n')
    print(count)
    f.close()

def test_in_corpus(model):
    #test whether the two words are in the word2vec dict
    for file in ['synonyms_max.txt', 'synonyms.txt', 'antonyms.txt', 'irrelevent_min2.txt']:
        f = open(file, 'r')
        j=[0,0]
        for i in f:

            j[0]+=1
            word1, word2 = i.strip().split(', ')
            if word1 in model and word2 in model:
                j[1]+=1
        f.close()
        print("In %r, %r of %r are in the corpus.\n" %(file, j[1], j[0]))


def save_word2vec():
    word2vec_dict = {}
    with open('../../embeddings/glove.6B.50d.txt', encoding='utf-8') as f:
        for line in f.readlines():
            line = line.rstrip().split()
            word = line[0]
            vec = np.asarray([float(i) for i in line[1:]])
            word2vec_dict[word] = vec
    g = open('../../data/word2vec_dict.bin', 'wb')
    pickle.dump(word2vec_dict, g)
    g.close()

def transform_to_vec(model,word1, word2):
    #get two words as input, and transform it to a feature vector
    vec1, vec2 = model[word1], model[word2]
    return np.concatenate((vec1, vec2, vec1*vec2, np.abs(vec1-vec2), vec1+vec2))

def transform_to_mat(model):
    """
    Load all the pairs in three txt files, transform it to matrix.
    X: features, shape of (num_instances, num_feat)
       features is a concatenation of x1, x2, (x1*x2), |x1 - x2|, (x1+x2),
    y: labels, shape of (num_instances)
    """
    X = []
    y = []
    for file in ['synonyms.txt', 'antonyms.txt', 'irrelevent_min2.txt']:
        f = open(file, 'r')
        j=[0,0]
        for i in f:
            j[0]+=1
            word1, word2 = i.strip().split(', ')
            if word1 in model and word2 in model:
                j[1]+=1
                X.append(transform_to_vec(model, word1, word2))
                if file == 'synonyms.txt':
                    y.append(1)
                else:
                    y.append(-1)
        f.close()
        print("In %r, %r of %r are in the corpus.\n" %(file, j[1], j[0]))
    X, y = np.array(X), np.array(y)
    print(str(X.shape))
    print(str(y.shape))
    return X, y


def main():
    save_word2vec()
    #generate_antonyms()
    generate_synonyms()
    generate_irrelevant()
    word2vec_dict = pickle.load(open('../../data/word2vec_dict.bin', 'rb'))
    test_in_corpus(word2vec_dict)

    X, y = transform_to_mat(word2vec_dict)
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.67, random_state=3)
    pickle.dump((X_train.astype(np.float64), X_test.astype(np.float64), y_train, y_test),open('word_mat_min.bin', 'wb'))

main()


import numpy as np
from sklearn.feature_extraction.text import CountVectorizer

from treform.weighting.iqf_qf_icf import iqf_qf_icf
from treform.weighting.tf_bdc import tf_bdc
from treform.weighting.tf_chi import tf_chi
from treform.weighting.tf_dc import tf_dc
from treform.weighting.tf_eccd import tf_eccd
from treform.weighting.tf_idf import tf_idf
from treform.weighting.tf_ig import tf_ig
from treform.weighting.tf_rf import tf_rf


class BaseWeighting:
    IN_TYPE = [str]
    OUT_TYPE = [list, str]

    def __init__(self):
        self.package = {}
        self.package["voca"] = []
        self.package["labelset"] = []
        self.package["vocafreq"] = {}
        self.package["weights"] = {}
        self.package["doclist"] = []
        self.package["docname"] = set()

    def preprocess(self, input, label_list=[]):
        vocafreq = self.package["vocafreq"]
        labelset = self.package["labelset"]
        corpus = []
        doccount = {}

        label = 'UNI'
        for i, _doc in enumerate(input):
            document = {}
            if len(label_list) > 0:
                document["label"] = label_list[i]
                label = label_list[i]
            else:
                document["label"] = label
            if label not in doccount:
                doccount[label] = 0
            doccount[label] += 1
            docname = document["label"] + str(doccount[label])  # label + str(doccount[label])
            #docname = label + str(doccount[label])
            document["document"] = docname

            document["split_sentence"] = _doc

            if label not in labelset:
                labelset.append(label)

            document["length"] = len(_doc)
            corpus.append(document)

        cv = CountVectorizer()
        cv_fit = cv.fit_transform(input)
        word_list = cv.get_feature_names_out()

        count_list = np.asarray(cv_fit.sum(axis=0))[0]
        vocabfreq = dict(zip(word_list, count_list))
        self.package["vocafreq"] = vocabfreq
        self.package["voca"] = word_list
        self.package["labelset"] = labelset

        #print(vocabfreq)

        return corpus

class TfIdf(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_idf(self.corpus, self.package)
        print('----------tf-idf--------')
        return weights

class TfBdc(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_bdc(self.corpus, self.package)
        print('----------tf-dbc--------')
        return weights

class IqfQfIcf(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = iqf_qf_icf(self.corpus, self.package)
        print('----------iqf-qf-icf--------')
        return weights

class TfChi(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_chi(self.corpus, self.package)
        print('----------tf-chi--------')
        return weights

class TfDc(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_dc(self.corpus, self.package)
        print('----------tf-dc--------')
        return weights

class TfEccd(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_eccd(self.corpus, self.package)
        print('----------tf-eccd--------')
        return weights

class TfIg(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_ig(self.corpus, self.package)
        print('----------tf-ig--------')
        return weights

class TfRf(BaseWeighting):
    def __init__(self, input, label_list=[]):
        super().__init__()
        self.corpus = super().preprocess(input, label_list=label_list)

    def __call__(self):
        weights = tf_rf(self.corpus, self.package)
        print('----------tf-rf--------')
        return weights


import argparse
import numpy as np

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--vocab_file', default='vocab.txt', type=str)
    parser.add_argument('--vectors_file', default='vectors.txt', type=str)
    args = parser.parse_args()

    with open(args.vocab_file, 'r') as f:
        words = [x.rstrip().split(' ')[0] for x in f.readlines()]
    with open(args.vectors_file, 'r') as f:
        vectors = {}
        for line in f:
            vals = line.rstrip().split(' ')
            vectors[vals[0]] = [float(x) for x in vals[1:]]

    vocab_size = len(words)
    vocab = {w: idx for idx, w in enumerate(words)}
    ivocab = {idx: w for idx, w in enumerate(words)}

    vector_dim = len(vectors[ivocab[0]])
    W = np.zeros((vocab_size, vector_dim))
    for word, v in vectors.items():
        if word == '<unk>':
            continue
        W[vocab[word], :] = v

    # normalize each word vector to unit length
    W_norm = np.zeros(W.shape)
    d = (np.sum(W ** 2, 1) ** (0.5))
    W_norm = (W.T / d).T
    evaluate_vectors(W_norm, vocab)

def evaluate_vectors(W, vocab):
    """Evaluate the trained word vectors on a variety of tasks"""

    filenames = [
        'capital-common-countries.txt', 'capital-world.txt', 'currency.txt',
        'city-in-state.txt', 'family.txt', 'gram1-adjective-to-adverb.txt',
        'gram2-opposite.txt', 'gram3-comparative.txt', 'gram4-superlative.txt',
        'gram5-present-participle.txt', 'gram6-nationality-adjective.txt',
        'gram7-past-tense.txt', 'gram8-plural.txt', 'gram9-plural-verbs.txt',
        ]
    prefix = './eval/question-data/'

    # to avoid memory overflow, could be increased/decreased
    # depending on system and vocab size
    split_size = 100

    correct_sem = 0; # count correct semantic questions
    correct_syn = 0; # count correct syntactic questions
    correct_tot = 0 # count correct questions
    count_sem = 0; # count all semantic questions
    count_syn = 0; # count all syntactic questions
    count_tot = 0 # count all questions
    full_count = 0 # count all questions, including those with unknown words

    for i in range(len(filenames)):
        with open('%s/%s' % (prefix, filenames[i]), 'r') as f:
            full_data = [line.rstrip().split(' ') for line in f]
            full_count += len(full_data)
            data = [x for x in full_data if all(word in vocab for word in x)]

        if len(data) == 0:
            print("ERROR: no lines of vocab kept for %s !" % filenames[i])
            print("Example missing line:", full_data[0])
            continue

        indices = np.array([[vocab[word] for word in row] for row in data])
        ind1, ind2, ind3, ind4 = indices.T

        predictions = np.zeros((len(indices),))
        num_iter = int(np.ceil(len(indices) / float(split_size)))
        for j in range(num_iter):
            subset = np.arange(j*split_size, min((j + 1)*split_size, len(ind1)))

            pred_vec = (W[ind2[subset], :] - W[ind1[subset], :]
                +  W[ind3[subset], :])
            #cosine similarity if input W has been normalized
            dist = np.dot(W, pred_vec.T)

            for k in range(len(subset)):
                dist[ind1[subset[k]], k] = -np.Inf
                dist[ind2[subset[k]], k] = -np.Inf
                dist[ind3[subset[k]], k] = -np.Inf

            # predicted word index
            predictions[subset] = np.argmax(dist, 0).flatten()

        val = (ind4 == predictions) # correct predictions
        count_tot = count_tot + len(ind1)
        correct_tot = correct_tot + sum(val)
        if i < 5:
            count_sem = count_sem + len(ind1)
            correct_sem = correct_sem + sum(val)
        else:
            count_syn = count_syn + len(ind1)
            correct_syn = correct_syn + sum(val)

        print("%s:" % filenames[i])
        print('ACCURACY TOP1: %.2f%% (%d/%d)' %
            (np.mean(val) * 100, np.sum(val), len(val)))

    print('Questions seen/total: %.2f%% (%d/%d)' %
        (100 * count_tot / float(full_count), count_tot, full_count))
    print('Semantic accuracy: %.2f%%  (%i/%i)' %
        (100 * correct_sem / float(count_sem), correct_sem, count_sem))
    print('Syntactic accuracy: %.2f%%  (%i/%i)' %
        (100 * correct_syn / float(count_syn), correct_syn, count_syn))
    print('Total accuracy: %.2f%%  (%i/%i)' % (100 * correct_tot / float(count_tot), correct_tot, count_tot))


if __name__ == "__main__":
    main()



import numpy as np
import scipy.sparse as sp
from sklearn.feature_extraction.text import CountVectorizer

class Cooccurrence(CountVectorizer):
    """Co-ocurrence matrix
    Convert collection of raw documents to word-word co-ocurrence matrix

    Parameters
    ----------
    encoding : string, 'utf-8' by default.
        If bytes or files are given to analyze, this encoding is used to
        decode.

    ngram_range : tuple (min_n, max_n)
        The lower and upper boundary of the range of n-values for different
        n-grams to be extracted. All values of n such that min_n <= n <= max_n
        will be used.

    max_df: float in range [0, 1] or int, default=1.0

    min_df: float in range [0, 1] or int, default=1

    Example
    -------

    >> import Cooccurrence
    >> docs = ['this book is good',
               'this cat is good',
               'cat is good shit']
    >> model = Cooccurrence()
    >> Xc = model.fit_transform(docs)

    Check vocabulary by printing
    >> model.vocabulary_

    """

    def __init__(self, encoding='utf-8', ngram_range=(1, 1),
                 max_df=1.0, min_df=1, max_features=None,
                 stop_words=None, normalize=True, vocabulary=None):

        super(Cooccurrence, self).__init__(
            ngram_range=ngram_range,
            max_df=max_df,
            min_df=min_df,
            max_features=max_features,
            stop_words=stop_words,
            vocabulary=vocabulary
        )

        self.X = None

        self.normalize = normalize

    def fit_transform(self, raw_documents, y=None):
        """Fit cooccurrence matrix

        Parameters
        ----------
        raw_documents : iterable
            an iterable which yields either str, unicode or file objects

        Returns
        -------
        Xc : Cooccurrence matrix

        """
        X = super(Cooccurrence, self).fit_transform(raw_documents)
        self.X = X

        n_samples, n_features = X.shape

        Xc = (X.T * X)
        if self.normalize:
            g = sp.diags(1./Xc.diagonal())
            Xc = g * Xc
        else:
            Xc.setdiag(0)

        return Xc

    def vocab(self):
        tuples = super(Cooccurrence, self).get_feature_names_out()
        vocabulary=[]
        for e_tuple in tuples:
            tokens = e_tuple.split()
            for t in tokens:
                if t not in vocabulary:
                    vocabulary.append(t)

        return vocabulary

    def word_histgram(self):
        word_list = super(Cooccurrence, self).get_feature_names()
        count_list = self.X.toarray().sum(axis=0)
        return dict(zip(word_list,count_list))


import logging
import os
import sys
import warnings
from os.path import expanduser
import os
import json

from treform.utility.base_util import db_hostname, is_my_pc
from treform.utility.log_util import LogUtil

class Config():
    def __init__(self, json_file):
        warnings.simplefilter(action='ignore', category=FutureWarning)  # ignore future warnings

        self.log = None
        if self.log is None:
            if len(sys.argv) == 1:  # by Pycharm or console
                if is_my_pc():  # my pc (pycharm client, mac)
                    self.log = LogUtil.get_logger(None, level=logging.DEBUG, console_mode=True)  # global log
                else:  # gpu pc (batch job, ubuntu)
                    self.log = LogUtil.get_logger(sys.argv[0], level=logging.DEBUG, console_mode=True)  # global log # console_mode=True for jupyter
            else:  # by batch script
                self.log = LogUtil.get_logger(sys.argv[0], level=logging.INFO, console_mode=False)  # global log

        #json_file = '/resources/korean_spelling_config.json'
        with open(json_file, encoding='utf-8') as f:
            config = json.load(f)

        self.HOME_DIR = config["HOME_DIR"]
        self.log.info('HOME_DIR: %s' % self.HOME_DIR)

        self.SPELLING_MODEL_DIR = config["SPELLING_MODEL_DIR"]

        self.PROJECT_DIR = os.path.join(self.HOME_DIR, '')

        self.DATA_DIR = os.path.join(self.PROJECT_DIR, 'data')
        self.log.info('DATA_DIR: %s' % self.DATA_DIR)
        if not os.path.exists(self.DATA_DIR):
            os.mkdir(self.DATA_DIR)

        self.MODELS_DIR = os.path.join(self.PROJECT_DIR, 'models')
        self.log.info('MODELS_DIR: %s' % self.MODELS_DIR)
        if not os.path.exists(self.MODELS_DIR):
            os.mkdir(self.MODELS_DIR)

        #################################################
        # tensorboard log dir
        #################################################
        self.TENSORBOARD_LOG_DIR = os.path.join(self.HOME_DIR, 'tensorboard_log')
        # log.info('TENSORBOARD_LOG_DIR: %s' % TENSORBOARD_LOG_DIR)
        if not os.path.exists(self.TENSORBOARD_LOG_DIR):
            os.mkdir(self.TENSORBOARD_LOG_DIR)

        #################################################
        # mnist
        #################################################
        self.MNIST_DIR = os.path.join(self.HOME_DIR, 'workspace', 'nlp4kor_tensorflow-mnist')
        self.MNIST_DATA_DIR = os.path.join(self.MNIST_DIR, 'data')
        self.MNIST_CNN_MODEL_DIR = os.path.join(self.MNIST_DIR, 'models', 'cnn')
        self.MNIST_DAE_MODEL_DIR = os.path.join(self.MNIST_DIR, 'models', 'dae')

        #################################################
        # ko.wikipedia.org
        #################################################
        self.KO_WIKIPEDIA_ORG_DIR = os.path.join(self.HOME_DIR, 'spelling_training')

        self.KO_WIKIPEDIA_ORG_INFO_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.info.txt')
        self.KO_WIKIPEDIA_ORG_URLS_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.urls.txt')
        self.KO_WIKIPEDIA_ORG_CHARACTERS_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.characters')

        self.KO_WIKIPEDIA_ORG_SENTENCES_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.sentences.gz')
        self.KO_WIKIPEDIA_ORG_TRAIN_SENTENCES_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.train.sentences.gz')
        self.KO_WIKIPEDIA_ORG_VALID_SENTENCES_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.valid.sentences.gz')
        self.KO_WIKIPEDIA_ORG_TEST_SENTENCES_FILE = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'ko.wikipedia.org.test.sentences.gz')

        self.KO_WIKIPEDIA_ORG_SPELLING_ERROR_CORRECTION_MODEL_DIR = os.path.join(self.KO_WIKIPEDIA_ORG_DIR, 'models', 'spelling_error_correction')

        #################################################
        # ko.wikipedia.org
        #################################################
        self.WIKIPEDIA_DIR = config["WIKIPEDIA_DIR"]

        # text (with string)
        #WIKIPEDIA_DATA_DIR = os.path.join(WIKIPEDIA_DIR, 'data')
        self.WIKIPEDIA_DATA_DIR = self.WIKIPEDIA_DIR
        if not os.path.exists(self.WIKIPEDIA_DATA_DIR):
            os.mkdir(self.WIKIPEDIA_DATA_DIR)

        # info
        self.WIKIPEDIA_INFO_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.info.txt')
        self.WIKIPEDIA_URLS_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.urls.txt')

        self.WIKIPEDIA_CHARACTERS_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.characters')
        self.WIKIPEDIA_SENTENCES_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.sentences.gz')

        self.WIKIPEDIA_TRAIN_SENTENCES_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.train.sentences.gz')
        self.WIKIPEDIA_VALID_SENTENCES_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.valid.sentences.gz')
        self.WIKIPEDIA_TEST_SENTENCES_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.test.sentences.gz')

        # csv (with character id)
        self.WIKIPEDIA_TRAIN_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.train.sentences.cid.gz')
        self.WIKIPEDIA_VALID_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.valid.sentences.cid.gz')
        self.WIKIPEDIA_TEST_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.test.sentences.cid.gz')

        self.WIKIPEDIA_NE_FILE = os.path.join(self.WIKIPEDIA_DATA_DIR, 'ko.wikipedia.org.sentences.ne.gz')


import argparse
import numpy as np
import sys

def generate():
    parser = argparse.ArgumentParser()
    parser.add_argument('--vocab_file', default='vocab.txt', type=str)
    parser.add_argument('--vectors_file', default='vectors.txt', type=str)
    args = parser.parse_args()

    with open(args.vocab_file, 'r') as f:
        words = [x.rstrip().split(' ')[0] for x in f.readlines()]
    with open(args.vectors_file, 'r') as f:
        vectors = {}
        for line in f:
            vals = line.rstrip().split(' ')
            vectors[vals[0]] = [float(x) for x in vals[1:]]

    vocab_size = len(words)
    vocab = {w: idx for idx, w in enumerate(words)}
    ivocab = {idx: w for idx, w in enumerate(words)}

    vector_dim = len(vectors[ivocab[0]])
    W = np.zeros((vocab_size, vector_dim))
    for word, v in vectors.items():
        if word == '<unk>':
            continue
        W[vocab[word], :] = v

    # normalize each word vector to unit variance
    W_norm = np.zeros(W.shape)
    d = (np.sum(W ** 2, 1) ** (0.5))
    W_norm = (W.T / d).T
    return (W_norm, vocab, ivocab)


def distance(W, vocab, ivocab, input_term):
    vecs = {}
    if len(input_term.split(' ')) < 3:
        print("Only %i words were entered.. three words are needed at the input to perform the calculation\n" % len(input_term.split(' ')))
        return 
    else:
        for idx, term in enumerate(input_term.split(' ')):
            if term in vocab:
                print('Word: %s  Position in vocabulary: %i' % (term, vocab[term]))
                vecs[idx] = W[vocab[term], :] 
            else:
                print('Word: %s  Out of dictionary!\n' % term)
                return

        vec_result = vecs[1] - vecs[0] + vecs[2]
        
        vec_norm = np.zeros(vec_result.shape)
        d = (np.sum(vec_result ** 2,) ** (0.5))
        vec_norm = (vec_result.T / d).T

        dist = np.dot(W, vec_norm.T)

        for term in input_term.split(' '):
            index = vocab[term]
            dist[index] = -np.Inf

        a = np.argsort(-dist)[:N]

        print("\n                               Word       Cosine distance\n")
        print("---------------------------------------------------------\n")
        for x in a:
            print("%35s\t\t%f\n" % (ivocab[x], dist[x]))


if __name__ == "__main__":
    N = 100;          # number of closest words that will be shown
    W, vocab, ivocab = generate()
    while True:
        input_term = raw_input("\nEnter three words (EXIT to break): ")
        if input_term == 'EXIT':
            break
        else:
            distance(W, vocab, ivocab, input_term)





import requests
import random
from bs4 import BeautifulSoup
import time
from selenium import webdriver
from selenium.webdriver.chrome.webdriver import WebDriver

class NaverBlog():
    def __init__(self):
        self.ab_url=[]
        self.query = ''

    def get_link(self, query, s_date, e_date):
        self.query = query
        self.s_date=s_date
        self.e_date=e_date
        options = webdriver.ChromeOptions()
        options.add_argument(
            "user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36")

        options.add_argument('headless')
        options.add_argument('--disable-gpu')
        options.add_argument('lang=ko_KR')
        browser = WebDriver(executable_path='D:\python_workspace\pyTextMiner\selenium_server\chromedriver.exe', options=options)
        #browser = WebDriver(executable_path='/usr/lib/chromium-browser/chromedriver', options=options)
        url = "https://m.search.naver.com/search.naver?where=m_blog&sm=mtb_opt&query=" + query + "&display=15&st=sim&nso=p%3Afrom" + s_date + "to" + e_date
        browser.get(url)
        browser.implicitly_wait(random.randrange(5,10))
        SCROLL_PAUSE_TIME = 1.5
        # Get scroll height
        last_height = browser.execute_script("return document.body.scrollHeight")
        while True:
            # Scroll down to bottom
            browser.execute_script("window.scrollTo(0, document.body.scrollHeight);")
            # Wait to load page
            time.sleep(SCROLL_PAUSE_TIME)
            # Calculate new scroll height and compare with last scroll height
            new_height = browser.execute_script("return document.body.scrollHeight")
            if new_height == last_height:
                cont = browser.page_source
                soup = BeautifulSoup(cont, 'html.parser')
                for urls in soup.select(".total_dsc"):
                    if urls["href"].startswith("https://m.blog.naver.com") or 'blog.me' in urls["href"]:
                        self.ab_url.append(urls['href'])
                break
            last_height = new_height
        time.sleep(random.randrange(5,15))

    def get_detail(self):
        f = open(self.query + "_" + self.s_date + "_" + self.e_date + '.txt', 'w', encoding='utf-8')
        for url in self.ab_url:
            print(url)
            time.sleep(random.randrange(10, 15))
            header = {
                'User-Agent':'user-agent=Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.87 Safari/537.36',
            }
            breq = requests.get(url, headers=header)
            time.sleep(random.randrange(20,30))
            cont = breq.content
            bsoup = BeautifulSoup(cont, 'html.parser')
            title = bsoup.select('.se-module.se-module-text.se-title-text')
            if title ==[]:
                title=bsoup.select('.tit_h3')
                if title == []:
                    title = bsoup.select('.se_textarea')
                    title2 = title[0].text.strip().replace('\n', " ").replace("\r", " ")
                    f.write(title2 + '\t')
                    print(title2)
                title2=title[0].text.strip().replace('\n', " ").replace("\r", " ")
                f.write(title2+'\t')
                print(title2)
            else:
                title2 = title[0].text.strip().replace('\n', " ").replace("\r", " ")
                f.write(title2+'\t')
                print(title2)
            time.sleep(3)

            pdate = bsoup.select('.blog_date')
            if pdate==[]:
                pdate=bsoup.select('.se_date')
                pdate2=pdate[0].get_text()
                f.write(pdate2.strip()+'\t')
                print(pdate2)
            else:
                pdate2=pdate[0].get_text()
                f.write(pdate2.strip()+'\t')
                print(pdate2)

            user = bsoup.select('.blog_author')
            if user == []:
                user = bsoup.select('.se_author')
                user2 = user[0].a.get_text()
                f.write(user2.strip()+'\t')
                print(user2)
            else:
                user2 = user[0].a.get_text()
                f.write(user2.strip() + '\t')
                print(user2)

            _text = bsoup.select('.se-main-container')
            if _text==[]:
                _text=bsoup.select('.post_ct')
                if _text == []:
                    _text = bsoup.select('.se_textarea')
                    text2 = _text[0].get_text().replace('\n', " ").replace("\r", " ").replace("\t", " ")
                    f.write(text2 + '\t')
                    print(text2.strip())
                text2 = _text[0].get_text().replace('\n', " ").replace("\r", " ").replace("\t", " ")
                f.write(text2+'\t')
                print(text2.strip())
            else:
                text2=_text[0].get_text().replace('\n', " ").replace("\r", " ").replace("\t", " ")
                f.write(text2+'\t')
                print(text2.strip())

            f.write(url+'\n')
        f.close()

if __name__ == '__main__':
    query = '우울증'
    s_date = "2017.01.01"
    e_date = "2017.01.31"

    s_from = s_date.replace(".", "")
    e_to = e_date.replace(".", "")

    blog = NaverBlog()
    blog.get_link(query,s_from,e_to)
    blog.get_detail()


import sys
# !{sys.executable} -m spacy download en
import re, numpy as np, pandas as pd
from pprint import pprint

# Gensim
import gensim, spacy, logging, warnings
import gensim.corpora as corpora
from gensim.utils import lemmatize, simple_preprocess
from gensim.models import CoherenceModel
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors

# NLTK Stop words
from matplotlib.patches import Rectangle
from nltk.corpus import stopwords
stop_words = stopwords.words('english')
stop_words.extend(['from', 'subject', 're', 'edu', 'use', 'not', 'would', 'say', 'could', '_', 'be', 'know', 'good', 'go', 'get', 'do', 'done', 'try', 'many', 'some', 'nice', 'thank', 'think', 'see', 'rather', 'easy', 'easily', 'lot', 'lack', 'make', 'want', 'seem', 'run', 'need', 'even', 'right', 'line', 'even', 'also', 'may', 'take', 'come'])

logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.ERROR)

# Import Dataset
df = pd.read_json('https://raw.githubusercontent.com/selva86/datasets/master/newsgroups.json')
df = df.loc[df.target_names.isin(['soc.religion.christian', 'rec.sport.hockey', 'talk.politics.mideast', 'rec.motorcycles']) , :]
print(df.shape)  #> (2361, 3)
df.head()

def sent_to_words(sentences):
    for sent in sentences:
        sent = re.sub('\S*@\S*\s?', '', sent)  # remove emails
        sent = re.sub('\s+', ' ', sent)  # remove newline chars
        sent = re.sub("\'", "", sent)  # remove single quotes
        sent = gensim.utils.simple_preprocess(str(sent), deacc=True)
        yield(sent)

# Convert to list
data = df.content.values.tolist()
data_words = list(sent_to_words(data))
print(data_words[:1])
# [['from', 'irwin', 'arnstein', 'subject', 're', 'recommendation', 'on', 'duc', 'summary', 'whats', 'it', 'worth', 'distribution', 'usa', 'expires', 'sat', 'may', 'gmt', ...trucated...]]

# Build the bigram and trigram models
bigram = gensim.models.Phrases(data_words, min_count=5, threshold=100) # higher threshold fewer phrases.
trigram = gensim.models.Phrases(bigram[data_words], threshold=100)
bigram_mod = gensim.models.phrases.Phraser(bigram)
trigram_mod = gensim.models.phrases.Phraser(trigram)

# !python3 -m spacy download en  # run in terminal once
def process_words(texts, stop_words=stop_words, allowed_postags=['NOUN', 'ADJ', 'VERB', 'ADV']):
    """Remove Stopwords, Form Bigrams, Trigrams and Lemmatization"""
    texts = [[word for word in simple_preprocess(str(doc)) if word not in stop_words] for doc in texts]
    texts = [bigram_mod[doc] for doc in texts]
    texts = [trigram_mod[bigram_mod[doc]] for doc in texts]
    texts_out = []
    nlp = spacy.load("en_core_web_sm")
    #nlp = spacy.load('en', disable=['parser', 'ner'])
    for sent in texts:
        doc = nlp(" ".join(sent))
        texts_out.append([token.lemma_ for token in doc if token.pos_ in allowed_postags])
    # remove stopwords once more after lemmatization
    texts_out = [[word for word in simple_preprocess(str(doc)) if word not in stop_words] for doc in texts_out]
    return texts_out

data_ready = process_words(data_words)  # processed Text Data!

# Create Dictionary
id2word = corpora.Dictionary(data_ready)

# Create Corpus: Term Document Frequency
corpus = [id2word.doc2bow(text) for text in data_ready]

# Build LDA model
lda_model = gensim.models.ldamodel.LdaModel(corpus=corpus,
                                           id2word=id2word,
                                           num_topics=4,
                                           random_state=100,
                                           update_every=1,
                                           chunksize=10,
                                           passes=10,
                                           alpha='symmetric',
                                           iterations=100,
                                           per_word_topics=True)

pprint(lda_model.print_topics())

def sentences_chart(lda_model=lda_model, corpus=corpus, start = 0, end = 13):
    corp = corpus[start:end]
    mycolors = [color for name, color in mcolors.TABLEAU_COLORS.items()]

    fig, axes = plt.subplots(end-start, 1, figsize=(20, (end-start)*0.95), dpi=160)
    axes[0].axis('off')
    for i, ax in enumerate(axes):
        if i > 0:
            corp_cur = corp[i-1]
            topic_percs, wordid_topics, wordid_phivalues = lda_model[corp_cur]

            print(wordid_topics)
            word_dominanttopic = [(lda_model.id2word[wd], topic[0]) for wd, topic in wordid_topics]
            ax.text(0.01, 0.5, "Doc " + str(i-1) + ": ", verticalalignment='center',
                    fontsize=16, color='black', transform=ax.transAxes, fontweight=700)

            #print(word_dominanttopic)
            # Draw Rectange
            topic_percs_sorted = sorted(topic_percs, key=lambda x: (x[1]), reverse=True)
            ax.add_patch(Rectangle((0.0, 0.05), 0.99, 0.90, fill=None, alpha=1,
                                   color=mycolors[topic_percs_sorted[0][0]], linewidth=2))

            word_pos = 0.06
            for j, (word, topics) in enumerate(word_dominanttopic):
                if j < 14:
                    ax.text(word_pos, 0.5, word,
                            horizontalalignment='left',
                            verticalalignment='center',
                            fontsize=16, color=mycolors[topics],
                            transform=ax.transAxes, fontweight=700)
                    word_pos += .009 * len(word)  # to move the word for the next iter
                    ax.axis('off')
            ax.text(word_pos, 0.5, '. . .',
                    horizontalalignment='left',
                    verticalalignment='center',
                    fontsize=16, color='black',
                    transform=ax.transAxes)

    plt.subplots_adjust(wspace=0, hspace=0)
    plt.suptitle('Sentence Topic Coloring for Documents: ' + str(start) + ' to ' + str(end-2), fontsize=22, y=0.95, fontweight=700)
    plt.tight_layout()
    plt.show()

sentences_chart(lda_model=lda_model, corpus=corpus)




import codecs
import gzip
import os.path
import subprocess

from treform.utility.base_util import is_windows_os
from treform.utility.num_util import NumUtil


class FileUtil(object):
    def __init__(self, data=None):
        self.data = data

    def __repr__(self):
        return self.data

    def read(self, file_path, is_binary=False):
        self.data = FileUtil.reads(file_path, is_binary)
        return self.data

    def write(self, file_path, is_binary=False):
        FileUtil.writes(self.data, file_path, is_binary)

    @staticmethod
    def delete(file_path):
        if os.path.exists(file_path):
            os.remove(file_path)

    @staticmethod
    def encoding(file_path):
        try:
            #return chardet.detect(open(file_path).read())['encoding']
            return None
        except:
            return None

    @staticmethod
    def reads(file_path, is_binary=False):
        if is_binary:
            read_mode = 'rb'
            with open(file_path, mode=read_mode) as f:
                data = ''.join([line for line in f.readlines()])
            return data
        else:  # text data
            read_mode = 'r'
            charset = FileUtil.encoding(file_path)
            with codecs.open(file_path, mode=read_mode, encoding=charset) as f:
                data = ''.join([line for line in f.readlines()])
            return data

    @staticmethod
    def writes(data, file_path, is_binary=False, encoding='UTF-8'):
        d = os.path.dirname(file_path)
        if len(d) > 0 and not os.path.exists(d):
            os.makedirs(d)
        if is_binary:
            write_mode = 'wb'
            if data and len(data) > 0:
                with open(file_path, mode=write_mode) as f:
                    f.write(data)
        else:  # text data
            write_mode = 'w'
            if data and len(data) > 0:
                with codecs.open(file_path, mode=write_mode, encoding=encoding) as f:
                    f.write(data)

    @staticmethod
    def count_lines(filename_or_list):
        lines = 0
        if isinstance(filename_or_list, str):
            filename_list = [filename_or_list]
        else:
            filename_list = filename_or_list

        for filename in filename_list:
            if filename.endswith('.gz') or filename.endswith('.zip'):
                gzip_format = True
            else:
                gzip_format = False

            if is_windows_os():
                if gzip_format:
                    f = gzip.open(filename, 'rt', encoding='utf8')
                else:
                    f = open(filename, 'r', encoding='utf8')

                with f:
                    for _ in f.readlines():
                        lines += 1
            else:
                if gzip_format:
                    p = subprocess.Popen('gzip -cd %s | wc -l' % filename, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
                else:
                    p = subprocess.Popen(['wc', '-l', filename], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

                result, err = p.communicate()
                if p.returncode != 0:
                    raise IOError(err)
                lines += int(result.strip().split()[0])
        return lines

    @staticmethod
    def print_n_write(file, s):
        print(s)
        file.write('%s\n' % (s,))

    @staticmethod
    def to_filename(s):
        return s.replace('/', '').replace('"', "'")

    @staticmethod
    def to_filename_from_dict(di, delimeter='_', include=None):
        if include is None:
            include = di.keys()
        return delimeter.join(
            ['{}={}'.format(key.replace(delimeter, ''), NumUtil.to_readable(val)) for key, val in sorted(di.items()) if
             key in include])

    @staticmethod
    def postfix(path, postfix):
        file_name, file_extension = os.path.splitext(path)
        return '%s%s%s' % (file_name, postfix, file_extension)


if __name__ == '__main__':
    name_max = subprocess.check_output("getconf NAME_MAX /", shell=True)
    path_max = subprocess.check_output("getconf PATH_MAX /", shell=True)
    print('name_max:', name_max)  # 255
    print('path_max:', path_max)  # 4096

    with open('output/file_test.txt', 'w') as file:
        FileUtil.print_n_write(file, 'aaa')
        FileUtil.print_n_write(file, ('a', 'b', 'c'))
        # print(FileUtil().read('/root/Documents/pn-a.txt'))
        # data = FileUtil.reads(__file__)
        # out_file_path = base_util.real_path('output/%s.bak' % os.path.basename(__file__))
        # FileUtil.writes(data, out_file_path, is_binary=False)
        # FileUtil().read(__file__).write(base_util.real_path('output/%s.bak' % __file__))


import nltk
from nltk.tokenize import sent_tokenize
from konlpy.tag import Mecab
import networkx as nx
import math
import matplotlib.pyplot as plt
import numpy as np
import re

WINDOW_SIZE = 10
STOP_POS = {
    'ko': ['IC', 'JKS', 'JKC', 'JKG', 'JKO', 'JKB', 'JKV', 'JKQ', 'JC', 'JX', 'XR', 'SF', 'SE', 'SSO', 'SSC', 'SC',
           'SY', 'EC', 'EF', 'ETN', 'ETM', 'XSV', 'XSA', 'XSN', 'XPN', 'NP'],
    'en': ['CC', 'CD', 'DT', 'EX', 'FW', 'IN', 'JJ', 'JJR', 'JJS', 'LS', 'MD', 'PDT', 'POS', 'PRP', 'PRP', 'RB', 'RBR',
           'RBS', 'RP', 'TO', 'UH', 'WDT', 'WP', 'WP', 'WRB', 'SL', 'SY', 'SN', 'VB', 'VBD', 'VBG', 'VBN', 'VBP',
           'VBZ']}  # 'NN','NNS','NNP','NNPS',


class TextRank:
    def __init__(self, pos_tagger_name='mecab', mecab_path='',
                 exceptional_stop_pos=[], lang='ko', stopwords=[]):
        self.stop_pos = STOP_POS[lang]
        # print(self.stop_pos)

        if pos_tagger_name == 'mecab':
            from konlpy.tag import Mecab
            self.pos_tagger = Mecab(mecab_path)
        elif pos_tagger_name == 'komoran':
            from konlpy.tag import Komoran
            self.pos_tagger = Komoran()
        elif pos_tagger_name == 'nltk':
            self.pos_tagger = None
        else:
            from konlpy.tag import Okt
            self.pos_tagger = Okt()

        if not exceptional_stop_pos:
            self.stop_pos = [x for x in self.stop_pos if x not in exceptional_stop_pos]

        self.stopwords = []
        if stopwords:
            self.stopwords = stopwords

        self.graph = nx.diamond_graph()
        self.graph.clear()  # 처음 생성시 graph에 garbage node가 남아있어 삭제

        self.tokens = []
    def keywords(self, text, n=10, window_size=WINDOW_SIZE):
        if self.pos_tagger is None:
            import nltk
            _tokens = nltk.word_tokenize(text)
            tokenized = nltk.pos_tag(_tokens)
        else:
            tokenized = self.pos_tagger.pos(text)
        # print(tokenized)

        nodes = []
        tokens = []
        for token in tokenized:
            if (len(token[0]) > 1) & (str(token[0]) not in self.stopwords) & (token[1][:3] not in self.stop_pos) & (
                    token[0] != token[1]):  # nltk는 특수문자는 품사가 태깅이 안되고 토큰이 그대로 품사에 나옴
                nodes.append(token)
            tokens.append(token)

        def connect(nodes, tokens):
            edges = []
            for window_start in range(0, (len(tokens) - window_size + 1)):
                window = tokens[window_start:window_start + window_size]

                for i in range(window_size):
                    for j in range(window_size):
                        if (i > j) & (window[i] in nodes) & (window[j] in nodes):
                            edges.append((window[i], window[j]))
            return edges

        graph = nx.diamond_graph()
        graph.clear()  # 처음 생성시 graph에 garbage node가 남아있어 삭제
        graph.add_nodes_from(list(set(nodes)))  # node 등록
        graph.add_edges_from(connect(nodes, tokens))  # edge 연결
        scores = nx.pagerank(graph)  # pagerank 계산
        rank = sorted(scores.items(), key=lambda x: x[1], reverse=True)  # score 역순 정렬
        return rank[:n]

    def build_keywords(self, text, window_size=WINDOW_SIZE):
        if self.pos_tagger is None:
            import nltk
            _tokens = nltk.word_tokenize(text)
            tokenized = nltk.pos_tag(_tokens)
        else:
            tokenized = self.pos_tagger.pos(text)

        # print(tokenized)
        nodes = []
        tokens = []
        for token in tokenized:
            if (len(token[0]) > 1) & (str(token[0]) not in self.stopwords) & (token[1][:3] not in self.stop_pos) & (
                    token[0] != token[1]):  # nltk는 특수문자는 품사가 태깅이 안되고 토큰이 그대로 품사에 나옴
                nodes.append(token)
            tokens.append(token)
            self.tokens.append(token[0])

        def connect(nodes, tokens):
            edges = []
            for window_start in range(0, (len(tokens) - window_size + 1)):
                window = tokens[window_start:window_start + window_size]

                for i in range(window_size):
                    for j in range(window_size):
                        if (i > j) & (window[i] in nodes) & (window[j] in nodes):
                            edges.append((window[i], window[j]))
            return edges

        self.graph.add_nodes_from(list(set(nodes)))  # node 등록
        self.graph.add_edges_from(connect(nodes, tokens))  # edge 연결

    def get_keywords(self, limit=10, combined_keywords=False):
        scores = nx.pagerank(self.graph)  # pagerank 계산
        rank = sorted(scores.items(), key=lambda x: x[1], reverse=True)  # score 역순 정렬

        _keywords = {}
        result = []
        for k in rank[:limit]:
            tuple = ()
            tuple = k[0][0],k[1],
            result.append(tuple)
            _keywords[k[0][0]] = k[1]

        keyphrases=self.get_combined_keywords(_keywords, self.tokens)
        #print('keyphrases ' + str(keyphrases))

        #we need to clear graph before a new document comes in
        self.graph.clear()

        if combined_keywords == True:
            return keyphrases

        return result

    def get_combined_keywords(self, _keywords, split_text):
        """
        :param keywords:dict of keywords:scores
        :param split_text: list of strings
        :return: combined_keywords:list
        """
        result = []
        _keywords = _keywords.copy()

        len_text = len(split_text)
        for i in range(len_text):
            word = self.strip_word(split_text[i])
            if word in _keywords:
                score = _keywords[word]
                combined_word = [word]
                if i + 1 == len_text:
                    result.append((word,score))  # appends last word if keyword and doesn't iterate
                for j in range(i + 1, len_text):
                    other_word = self.strip_word(split_text[j])
                    if other_word in _keywords and other_word == split_text[j] \
                            and other_word not in combined_word:
                        combined_word.append(other_word)
                    else:
                        for keyword in combined_word:
                            _keywords.pop(keyword)

                        result.append((" ".join(combined_word),score))
                        break

        return result

    def strip_word(self, word):
        stripped_word_list = list(self.tokenize(word))
        return stripped_word_list[0] if stripped_word_list else ""

    def tokenize(self, text, lowercase=False):
        PAT_ALPHABETIC = re.compile('(((?![\d])\w)+)', re.UNICODE)

        if lowercase:
            text = text.lower()
        for match in PAT_ALPHABETIC.finditer(text):
            yield match.group()

    def print_keywords(self, text, n=10, window_size=WINDOW_SIZE):
        print("Keyword : ")
        for k in self.keywords(text, n, window_size):
            print("{} - {}".format(k[0], k[1]))

    def summarize(self, text, max=3):
        # 자카드 유사도 계산
        def jaccard_similarity(query, document):
            intersection = set(query).intersection(set(document))
            union = set(query).union(set(document))
            return len(intersection) / len(union)

        # 문장간 유사도 측정 (BoW를 활용 코사인 유사도 측정)
        def sentence_similarity(sentence1, sentence2):
            sentence1 = [t[0] for t in self.pos_tagger.pos(sentence1) if t[1][:3] not in self.stop_pos]  # 개선 필요
            sentence2 = [t[0] for t in self.pos_tagger.pos(sentence2) if t[1][:3] not in self.stop_pos]  # 개선 필요
            # print(sentence1)
            return jaccard_similarity(sentence1, sentence2)

        def sentences(doc):
            return sent_tokenize(doc)  # [s[0].strip() for s in doc]

        def connect(doc):
            return [(start, end, sentence_similarity(start, end))
                    for start in doc for end in doc if start is not end]

        # tokens = self.pos_tagger(text)
        sentence_li = sentences(text)
        graph = nx.diamond_graph()
        graph.clear()  # 처음 생성시 graph에 garbage node가 남아있어 삭제
        graph.add_nodes_from(sentence_li)  # node 등록
        graph.add_weighted_edges_from(connect(sentence_li))  # edge 연결
        scores = nx.pagerank(graph)  # pagerank 계산

        rank = sorted(scores.items(), key=lambda x: x[1], reverse=True)  # score 역순 정렬
        return rank[:max]

    def print_summarize(self, text, n=3):
        print("Summarize : ")
        for s in self.summarize(text, n):
            print("{} - {}".format(s[0], s[1]))

    def get_keywords_list(self, text, keyword_length):
        keywords = self.keywords(text, keyword_length)
        keywords = [keyword[0][0] for keyword in keywords]
        return ' '.join(keywords)

    def get_summarization_list(self, text, summarization_length):
        return ' '.join([s[0] for s in self.summarize(text, summarization_length)])




from gensim import corpora, models, similarities
import pickle
import gensim

class tfidf:
    def __init__(self):
        name = 'tfidf'

    def createDictionary(self, text_data):
        dictionary = corpora.Dictionary(text_data)
        corpus = [dictionary.doc2bow(text) for text in text_data]

        pickle.dump(corpus, open('corpus.pkl', 'wb'))
        dictionary.save('dictionary.gensim')

        return corpus, dictionary

    def run(self, text_data):
        _corpus, dictionary = self.createDictionary(text_data)
        tf_idf = models.TfidfModel(_corpus)  # step 1 -- initialize a model
        corpus_tfidf = tf_idf[_corpus]
        for doc in corpus_tfidf:
            print(doc)

        lsi = models.LsiModel(corpus_tfidf, id2word=dictionary, num_topics=5)  # initialize an LSI transformation
        result = lsi.print_topics(5,20)
        for a_topic in result:
            print("LSI results " + str(a_topic))

        corpus_lsi = lsi[corpus_tfidf]  # create a double wrapper over the original corpus: bow->tfidf->fold-in-lsi
        #for doc in corpus_lsi: # both bow->tfidf and tfidf->lsi transformations are actually executed here, on the fly
            #print(doc)

if __name__ == '__main__':
    import pyTextMiner as ptm
    import io
    import nltk

    corpus = ptm.CorpusFromFile('../donald.txt')
    pipeline = ptm.Pipeline(ptm.splitter.NLTK(), ptm.tokenizer.Komoran(),
                            ptm.helper.POSFilter('NN*'),
                            ptm.helper.SelectWordOnly(),
                            ptm.helper.StopwordFilter(file='../stopwordsKor.txt'),
                            ptm.ngram.NGramTokenizer(3))

    result = pipeline.processCorpus(corpus)

    id = 0
    text_data = []
    for doc in result:
        new_doc = []
        for sent in doc:
            for _str in sent:
                if len(_str) > 0:
                    new_doc.append(_str)
        text_data.append(new_doc)
        id += 1

    tfidf().run(text_data)


import string
from collections import Counter
import os
from nltk import bigrams
from collections import defaultdict
import operator
import numpy as np
import platform

class BaseCooccurrence:
    INPUT=[list,str]
    OUTPUT=[list,tuple]

class CooccurrenceWorker(BaseCooccurrence):
    def __init__(self):
        name = 'cooccurrence'

        from sklearn.feature_extraction.text import CountVectorizer
        import treform.cooccurrence.cooccurrence as co
        self.inst = co.Cooccurrence(ngram_range=(2, 2), stop_words='english')

    def __call__(self, *args, **kwargs):
        # bigram_vectorizer = CountVectorizer(ngram_range=(1, 2), vocabulary={'awesome unicorns': 0, 'batman forever': 1})
        co_occurrences = self.inst.fit_transform(args[0])
        # print('Printing sparse matrix:', co_occurrences)
        # print(co_occurrences.todense())
        sum_occ = np.sum(co_occurrences.todense(), axis=0)
        # print('Sum of word-word occurrences:', sum_occ)

        # Converting itertor to set
        result = zip(self.inst.get_feature_names_out(), np.array(sum_occ)[0].tolist())
        result_set = list(result)
        return result_set, self.inst.vocab()

class CooccurrenceManager:
    def __init__(self):
        self.d = {}  # 단어->단어ID로 변환할때 사용
        self.w = []  # 단어ID->단어로 변환할 때 사용

    def getIdOrAdd(self, word):
        # 단어가 이미 사전에 등록된 것이면 해당하는 ID를 돌려주고
        if word in self.d: return self.d[word]
        # 그렇지 않으면 새로 사전에 등록하고 ID를 부여함
        self.d[word] = len(self.d)
        self.w.append(word)
        return len(self.d) - 1

    def getWord(self, id):
        return self.w[id]

    def calculateCooccurrence(self, list):
        count = {}  # 동시출현 빈도가 저장될 dict
        words = list(set(list))  # 단어별로 분리한 것을 set에 넣어 중복 제거하고, 다시 list로 변경
        wids = [self.getIdOrAdd(w) for w in words]
        for i, a in enumerate(wids):
            for b in wids[i + 1:]:
                if a == b: continue  # 같은 단어의 경우는 세지 않음
                if a > b: a, b = b, a  # A, B와 B, A가 다르게 세어지는것을 막기 위해 항상 a < b로 순서 고정
                count[a, b] = count.get((a, b), 0) + 1  # 실제로 센다

        sorted = []
        for tup in count:
            freq = count[tup]
            left_word = self.getWord(count[0])
            right_word = self.getWord(count[1])
            sorted.append(((left_word, right_word), freq))
        return sorted, words

    def computeCooccurence(self, list, target=''):
        com = defaultdict(lambda: defaultdict(int))
        count_all = Counter()
        count_all1 = Counter()

        uniqueList = []
        for _array in list:
            for line in _array:
                for word in line:
                    if len(target) < 1:
                        if word not in uniqueList:
                            uniqueList.append(word)

                terms_bigram = bigrams(line)
                # Update the counter
                count_all.update(line)
                count_all1.update(terms_bigram)

                # Build co-occurrence matrix
                for i in range(len(line) - 1):
                    for j in range(i + 1, len(line)):
                        w1, w2 = sorted([line[i], line[j]])
                        if w1 != w2:
                            com[w1][w2] += 1



        com_max = []
        # For each term, look for the most common co-occurrent terms
        for t1 in com:
            t1_max_terms = sorted(com[t1].items(), key=operator.itemgetter(1), reverse=True)[:5]
            for t2, t2_count in t1_max_terms:
                if len(target)>0 and (target is t1 or target is t2):
                    if t1 not in uniqueList:
                        uniqueList.append(t1)
                    if t2 not in uniqueList:
                        uniqueList.append(t2)
                    com_max.append(((t1, t2), t2_count))
        # Get the most frequent co-occurrences
        terms_max = sorted(com_max, key=operator.itemgetter(1), reverse=True)

        return terms_max, uniqueList


class CooccurrenceExternalManager():
    def __init__(self, program_path='./external_program', input_file='', output_file='', threshold=2, num_workers=3):
        print('running external program')
        self.program_path = program_path
        self.input_file = input_file
        self.output_file = output_file
        self.threshold = threshold
        self.num_workers = num_workers

        c_dir = os.getcwd()
        self.stdout = os.path.join(c_dir, 'log.txt')
        print(self.stdout)

    def execute(self):
        import subprocess
        import os
        os.chdir(self.program_path)
        print(os.getcwd())

        executable = ''
        if platform.system() == 'Windows':
            executable = 'ccount.exe '
        elif platform.system() == 'Darwin':
            executable = './ccount_mac '
        elif platform.system() == 'Linux':
            executable = './ccount '

        program = ' -i ' + self.input_file + ' -o ' + self.output_file + ' -w ' + str(self.num_workers) + ' --threshold ' + str(self.threshold)
        print(self.program_path + "\\"+ executable + " " + program)
        #subprocess.call(executable + program, cwd=self.program_path, stderr=subprocess.STDOUT,
        #                stdout=open(self.stdout, 'w'), shell=True)

        returned_output = subprocess.check_output(executable + " " + program)
        subprocess.run(executable + " " + program, shell=True, check=True)
        print(returned_output)
        print('finished computing co-occurrence.')




''' list(str) => list(tuple) '''

class BaseTagger:
    IN_TYPE = [list, str]
    OUT_TYPE = [list, tuple]

class NLTK(BaseTagger):
    def __init__(self):
        import nltk
        nltk.download('averaged_perceptron_tagger')

        from nltk.tag.perceptron import PerceptronTagger

        self.inst = PerceptronTagger()

    def __call__(self, *args, **kwargs):
        return self.inst.tag(args[0])


from treform.collector.colBasic import *
from treform.collector.colNaverNews import *
from treform.collector.colChosun import *
from treform.collector.colDonga import *
from treform.collector.colJoongang import *
from treform.collector.colBigkinds import *
from treform.collector.colNaverBlog import *